#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import GraphAction, cachedmethod, utils, SmewtException, Media
from smewt.base.utils import smewtDirectory
from smewt.guessers.guesser import Guesser
from smewt.media.series import Episode, Series
from smewt.base.mediaobject import foundMetadata
from pygoo import MemoryObjectGraph, Equal
from urllib import urlopen,  urlencode
from tvdbmetadataprovider import TVDBMetadataProvider
import logging

log = logging.getLogger('smewt.guessers.episodeimdb')


class EpisodeTVDB(GraphAction):

    def canHandle(self, query):
        if query.find_one(Media).type() not in [ 'video', 'subtitle' ]:
            raise SmewtException("%s: can only handle video or subtitle media objects" % self.__class__.__name__)

    def perform(self, query):
        self.checkValid(query)
        self.query = query

        ep = query.find_one(Episode)

        log.debug('EpisodeTvdb: finding more info on %s' % ep)
        if ep.get('series') is None:
            raise SmewtException("EpisodeTVDB: Episode doesn't contain 'series' field: %s" % ep)

        # little hack: if we have no season number, add 1 as default season number
        # (helps for series which have only 1 season)
        if ep.get('season') is None:
            ep.season = 1

        try:
            mdprovider = TVDBMetadataProvider()
            result = mdprovider.startEpisode(ep)

        except SmewtException, e:
            # series could not be found, return a dummy Unknown series instead so we can group them somewhere
            log.warning('Could not find series for file: %s' % query.find_one(Media).filename)
            noposter = smewtDirectory('smewt', 'media', 'common', 'images', 'noposter.png')
            result = MemoryObjectGraph()
            result.Series(title = 'Unknown', loresImage = noposter, hiresImage = noposter)

        # update the series
        query.delete_node(ep.series.node)
        ep.series = query.add_object(result.find_one(Series)) # this add_object should be unnecessary

        # and add all the potential episodes
        for ep in result.find_all(Episode):
            query.add_object(ep, recurse = Equal.OnLiterals)


        return query



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import Media, utils
from smewt.base.textutils import toUtf8
from smewt.media import Movie, Subtitle
from smewt.taggers.tagger import Tagger
from smewt.guessers import *
import logging

log = logging.getLogger('smewt.taggers.movietagger')

class MovieTagger(Tagger):
    def perform(self, query):
        log.info('MovieTagger tagging movie: %s' % toUtf8(query.find_one(Media).filename))
        filenameMetadata = MovieFilename().perform(query)
        filenameMovie = filenameMetadata.find_one(Movie)
        log.info('MovieTagger found info from filename: %s' % filenameMovie)
        result = MovieTMDB().perform(filenameMetadata)

        media = result.find_one(Media)
        if not media.metadata:
            log.warning('Could not find any tag for: %s' % media)

        # import the info we got from the filename if nothing better came in with MovieTMDB
        for prop in filenameMovie.keys():
            if prop not in media.metadata and prop not in media:
                media[prop] = filenameMovie[prop]

        # import subtitles correctly
        if media.type() == 'subtitle':
            # FIXME: problem for vobsubs: as a media points to a single metadata object, we cannot
            # represent a .sub for 3 different languages...
            subs = []
            for language in utils.guessCountryCode(media.filename):
                subs += [ result.Subtitle(metadata = media.metadata,
                                          language = language) ]

            media.metadata = subs


        self.cleanup(result)

        log.debug('Finished tagging: %s' % media)
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from collections import defaultdict

class SmewtDict(defaultdict):
    def __init__(self, schema):
        defaultdict.__init__(self, lambda x: None)
        self.schema = schema

    def __missing__(self, key):
        #if not key in self.schema:
        #    raise KeyError
        return None


class ValidatingSmewtDict(SmewtDict):
    def __init__(self, schema):
        super(ValidatingSmewtDict, self).__init__(schema)

    def __setitem__(self, key, value):
        if key in self.schema:
            # TODO: change this to an exception
            assert(value is None or isinstance(value, self.schema[key]))

        defaultdict.__setitem__(self, key, value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import re, logging
from urllib import urlopen, urlencode
from smewt.base import utils, SmewtException, cachedmethod
from pygoo import ontology, MemoryObjectGraph
from smewt.base.utils import curlget
from smewt.base.textutils import simpleMatch, between, levenshtein
from subtitleobject import Subtitle
from lxml import etree
ontology.import_classes([ 'Movie', 'Series', 'Episode' ])

log = logging.getLogger('TVSubtitlesProvider')


class TVSubtitlesProvider:

    baseUrl = 'http://www.tvsubtitles.net'

    @cachedmethod
    def getLikelySeriesUrl(self, name):
        data = urlencode({ 'q': name })
        html = etree.HTML(urlopen(self.baseUrl + '/search.php', data).read())
        matches = [ s.find('a') for s in html.findall(".//div[@style='']") ]

        # add baseUrl and remove year information
        result = []
        for match in matches:
            seriesID = int(match.get('href').split('-')[1].split('.')[0]) # remove potential season number
            seriesUrl = self.baseUrl + '/tvshow-%d.html' % seriesID
            title = match.text
            try:
                idx = title.find('(') - 1
                title = title[:idx]
            except: pass

            result.append({ 'title': title, 'url': seriesUrl })

        if not matches:
            raise SmewtException("Couldn't find any matching series for '%s'" % name)

        return result

    @cachedmethod
    def getSeriesID(self, name):
        # get most likely one if more than one found
        # FIXME: this hides another potential bug which is that tvsubtitles returns a lot of
        # false positives that it doesn't return when using from a "normal" webbrowser...
        urls = [ (levenshtein(s['title'], name), s) for s in self.getLikelySeriesUrl(name) ]
        url = sorted(urls)[0][1]['url']
        return int(simpleMatch(url, 'tvshow-(.*?).html'))

    @cachedmethod
    def getEpisodeID(self, series, season, number):
        seriesID = self.getSeriesID(series)
        seasonHtml = urlopen(self.baseUrl + '/tvshow-%d-%d.html' % (seriesID, season)).read()
        try:
            episodeRowHtml = between(seasonHtml, '%dx%02d' % (season, number), '</tr>')
        except IndexError:
            raise SmewtException("Season %d Episode %d unavailable for series '%s'" % (season, number, series))
        return int(simpleMatch(episodeRowHtml, 'episode-(.*?).html'))

    #@cachedmethod
    # cachedmethod doesn't work because of weakrefs in the graph...
    def getAvailableSubtitlesID(self, episode):
        log.debug('Getting subtitles id for %s %dx%2d' % (episode.series.title, episode.season, episode.episodeNumber))
        episodeID = self.getEpisodeID(episode.series.title, episode.season, episode.episodeNumber)
        episodeURL = self.baseUrl + '/episode-%d.html' % episodeID
        episodeHtml = curlget(episodeURL)
        subtitlesHtml = between(episodeHtml, 'Subtitles for this episode', 'Back to')

        result = MemoryObjectGraph()
        ep = result.Episode(episode)

        episodeHtml = between(episodeHtml, '<b>Subtitles for this episode:</b>', '<br clear=all>')
        ehtml = etree.HTML(episodeHtml)

        for slink, sub in zip(ehtml.findall('.//a'),
                              ehtml.findall(".//div[@class='subtitlen']")):
            result.Subtitle(metadata = ep,
                            language = simpleMatch(sub.find('.//img').get('src'), 'flags/(.*?).gif'),
                            tvsubid = int(between(slink.get('href'), '-', '.')),
                            title = (sub.find('.//h5').find('img').tail or '').strip(),
                            source = (sub.find(".//p[@title='rip']").find('img').tail or '').strip(),
                            release = (sub.find(".//p[@title='release']").find('img').tail or '').strip())

        return result


    def downloadSubtitle(self, basename, series, season, episode, language, videoFilename = None):
        """videoFilename is just used a hint when we find multiple subtitles"""
        import cStringIO, zipfile, os.path
        subs = [ sub for sub in self.getAvailableSubtitlesID(series, season, episode) if sub['language'] == language ]

        if not subs:
            return

        sub = subs[0]
        if len(subs) > 1:
            log.warning('More than 1 possible subtitle found: %s', str(subs))
            if videoFilename:
                dists = [ (levenshtein(videoFilename, sub['title']), sub) for sub in subs ]
                sub = sorted(dists)[0][1]
            log.warning('Choosing %s' % sub)

        cd = utils.CurlDownloader()
        # first get this page to get session cookies...
        cd.get(self.baseUrl + '/subtitle-%s.html' % sub['id'])
        # ...then grab the sub file
        cd.get(self.baseUrl + '/download-%s.html' % sub['id'])

        zf = zipfile.ZipFile(cStringIO.StringIO(cd.contents))
        filename = zf.infolist()[0].filename
        extension = os.path.splitext(filename)[1]
        subtext = zf.read(filename)
        resultFilename = basename + extension
        subf = open(resultFilename, 'w')
        subf.write(subtext)
        subf.close()
        return resultFilename

    # everything down from here is the implementation of the SubtitleProvider interface
    def canHandle(self, metadata):
        return isinstance(metadata, Episode)

    #def titleFilter(self, title):
    #    return lambda x: x.series.title == title

    def getAvailableSubtitles(self, metadata):
        try:
            if metadata.get('season') is None:
                raise SmewtException("'season' attribute not in object")
            if metadata.get('episodeNumber') is None:
                raise SmewtException("'episodeNumber' attribute not in object")
        except SmewtException, e:
            log.warning('TVSubtitlesProvider: %s' % str(e))
            return []

        return self.getAvailableSubtitlesID(metadata)


    def getSubtitle(self, subtitle):
        '''This method should return the contents of the given subtitle as a string.
        The Subtitle object should be picked among the ones returned by the getAvailableSubtitles
        method.

        If the subtitle could not be found, a SubtitleNotFoundError exception should be raised.'''
        import cStringIO, zipfile, os.path

        cd = utils.CurlDownloader()
        # first get this page to get session cookies...
        cd.get(self.baseUrl + '/subtitle-%d.html' % subtitle.tvsubid)
        # ...then grab the sub file
        cd.get(self.baseUrl + '/download-%d.html' % subtitle.tvsubid)

        zf = zipfile.ZipFile(cStringIO.StringIO(cd.contents))
        filename = zf.infolist()[0].filename
        #extension = os.path.splitext(filename)[1]
        subtext = zf.read(filename)

        return subtext


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import GraphAction, utils, textutils
from smewt.base import Media, Metadata, SmewtException
from smewt.base.utils import tolist
from smewt.media import Episode, Series
import logging, re

log = logging.getLogger('smewt.guessers.episodefilename')


class EpisodeFilename(GraphAction):

    supportedTypes = [ 'video', 'subtitle' ]

    def __init__(self):
        super(EpisodeFilename, self).__init__()

    def canHandle(self, query):
        if query.find_one(Media).type() not in [ 'video', 'subtitle' ]:
            raise SmewtException("%s: can only handle video or subtitle media objects" % self.__class__.__name__)

    def perform(self, query):
        self.checkValid(query)

        media = query.find_one(Media)

        log.debug('EpisodeFilename guesser working on file: %s' % media.filename)

        name = utils.splitPath(media.filename)
        filename = name[-1]

        # heuristic 1: try to guess the season & epnumber using S01E02 and 1x02 patterns
        sep = '[ \._-]'
        rexps = [ 'season (?P<season>[0-9]+)',
                  sep + '(?P<episodeNumber>[0-9]+)(?:v[23])?' + sep, # v2 or v3 for some mangas which have multiples rips
                  '(?P<season>[0-9]+)[x\.](?P<episodeNumber>[0-9]+)',
                  '[Ss](?P<season>[0-9]+) ?[Ee](?P<episodeNumber>[0-9]+)'
                  ]

        for n in name:
            for match in textutils.matchAllRegexp(n, rexps):
                log.debug('Found with confidence 1.0: %s' % match)
                ep = query.Episode(confidence = 1.0, allow_incomplete = True, **match)
                media.append('matches', ep)

        # cleanup a bit by removing unlikely eps numbers which are probably numbers in the title
        # or even dates in the filename, etc...
        niceGuess = None
        for md in tolist(media.get('matches')):
            if 'episodeNumber' in md and 'season' in md:
                niceGuess = md
            if 'episodeNumber' in md and 'season' not in md and md.episodeNumber > 1000:
                log.debug('Removing unlikely %s', str(md))
                query.delete_node(md.node)
        # if we have season+epnumber, remove single epnumber guesses
        if niceGuess:
            for md in query.find_all(node_type = Episode):
                if 'episodeNumber' in md and 'season' not in md:
                    log.debug('Removing %s because %s looks better' % (md, niceGuess))
                    query.delete_node(md.node)


        # heuristic 2: try to guess the serie title from the filename
        for rexp in rexps:
            result = re.compile(rexp, re.IGNORECASE).search(filename)
            if result:
                title = textutils.cleanString(filename[:result.span()[0]])
                log.debug('Found with confidence 0.6: series title = %s' % title)

                series = query.find_or_create(Series, title = title)
                media.append('matches', query.Episode(confidence = 0.6, allow_incomplete = True, series = series))


        # heuristic 3: try to guess the serie title from the parent directory!
        result = query.Episode(allow_incomplete = True)
        if textutils.matchAnyRegexp(name[-2], [ 'season (?P<season>[0-9]+)',
                                               # TODO: need to find a better way to have language packs for regexps
                                               'saison (?P<season>[0-9]+)' ]):
            log.debug('Found with confidence 0.8: series title = %s' % name[-3])
            s = query.find_or_create(Series, title = name[-3])
            result.series = s
            result.confidence = 0.8
            media.append('matches', result)

        else:
            log.debug('Found with confidence 0.4: series title = %s' % name[-2])
            s = query.find_or_create(Series, title = name[-2])
            result.series = s
            result.confidence = 0.4
            media.append('matches', result)

        # heuristic 4: add those anyway with very little probability, so that if don't find anything we can still use this
        media.append('matches', query.Episode(confidence = 0.1, series = query.Series(title = 'Unknown'), season = 1, episodeNumber = -1))

        # post-processing
        # we could already clean a bit the data here by solving it and comparing it to
        # each element we found, eg: remove all md which have an improbable episode number
        # such as 72 if some other valid episode number has been found, etc...

        # if the episode number is higher than 100, we assume it is season*100+epnumber
        for md in query.find_all(node_type = Episode, valid_node = lambda x: x.episodeNumber > 100):
            num = md.episodeNumber
            # it's the only guess we have, make it look like it's an episode
            # FIXME: maybe we should check if we already have an estimate for the season number?
            md.season = num // 100
            md.episodeNumber = num % 100


        return query



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#



# filename- and network-related functions
import sys, os, os.path, fnmatch,  errno
import pycurl
from PyQt4.QtCore import QSettings, QVariant
import smewt
from pygoo.utils import tolist, toresult

class MethodID(object):
    def __init__(self, filename, module, className, methodName):
        self.filename = filename
        self.module = module
        self.className = className
        self.methodName = methodName

    def __str__(self):
        return 'module: %s - class: %s - func: %s' % (self.module, self.className, self.methodName)

def callerid():
    f = sys._getframe(1)

    filename = f.f_code.co_filename
    module = ''
    className = ''

    try:
        module = f.f_locals['self'].__class__.__module__
        className = f.f_locals['self'].__class__.__name__
    except:
        pass

    methodName = f.f_code.co_name

    return MethodID(filename, module, className, methodName)

def currentPath():
    '''Returns the path in which the calling file is located.'''
    return os.path.dirname(os.path.join(os.getcwd(), sys._getframe(1).f_globals['__file__']))

def makedir(path):
    try:
        os.makedirs(path)
    except OSError, e:
        if e.errno == errno.EEXIST:
            pass
        else: raise

def pathToUrl(path):
    if sys.platform == 'win32':
        # perform some drive letter trickery
        return 'localhost/%s:/%s' % (path[0], path[3:])

    return path

def smewtDirectory(*args):
    return os.path.join(currentPath(), '..', '..', *args)

def smewtDirectoryUrl(*args):
    return pathToUrl(smewtDirectory(*args))

def smewtUserPath(*args):
    settings = QSettings()
    userdir = unicode(settings.value('user_dir').toString())
    if not userdir:
        if sys.platform == 'win32':
            userdir = os.path.join(os.environ['USERPROFILE'], 'Application Data', 'Smewt')
        else:
            userdir = os.path.dirname(unicode(settings.fileName()))

        settings.setValue('user_dir',  QVariant(userdir))

    return os.path.join(userdir, *args)

def smewtUserDirectory(*args):
    userdir = smewtUserPath(*args)
    makedir(userdir) # make sure directory exists

    return userdir


def smewtUserDirectoryUrl(*args):
    return pathToUrl(smewtUserDirectory(*args))


def splitPath(path):
    result = []
    while True:
        head, tail = os.path.split(path)
        if head == '/' and tail == '':
            return [ '/' ] + result
        if head == '' and tail == '':
            return result
        result = [ tail ] + result
        path = head

def commonRoot(pathlist):
    if not pathlist:
        return []

    root = splitPath(pathlist[0])
    for path in pathlist[1:]:
        for i, dir in enumerate(splitPath(path)):
            try:
                if root[i] != dir:
                    root = root[:i]
                    break
            except IndexError:
                break
        else:
            root = root[:len(splitPath(path))]

    return os.path.join(*root)

def parentDirectory(path):
    parentDir = splitPath(path)[:-1]
    return os.path.join(*parentDir)


def guessCountryCode(filename):
    '''Given a subtitle filename, tries to guess which languages it contains.
    As a subtitle file can contain multiple subtitles, this function returns a list
    of found languages.'''

    # try to guess language from filename
    langs = [ lang.lower() for lang in filename.split('.') ]

    languages = { 'english': 'en',
                  'french': 'fr',
                  'francais': 'fr',
                  u'fran\xe7ais': 'fr',
                  'spanish': 'es',
                  'espanol': 'es',
                  u'espa\xf1ol': 'es'
                  }

    if len(langs) >= 3:
        l = langs[-2].lower()
        for lang, code in languages.items():
            if l[-2:] == lang[:2] or l[-3:] == lang[:3] or l[-len(lang):] == lang:
                return [ code ]

    # try to look inside the .idx, if it exists
    langs = set()
    if os.path.exists(filename[:-3] + 'idx'):
        lines = open(filename[:-3] + 'idx').readlines()
        for l in lines:
            if l[:3] == 'id:':
                langs.add(l[4:6])

    if langs:
        return list(langs)

    return [ 'unknown' ]


def matchFile(filename, validFiles = ['*']):
    for validFile in validFiles:
        # if validFile is a string pattern, do filename matching
        if isinstance(validFile, basestring):
            if fnmatch.fnmatch(filename, validFile):
                return True
        else:
            # we assume it's a filter function that returns whether a file should be considered
            if validFile(filename):
                return True

    return False


def dirwalk(directory, validFiles = ['*'], recursive = True):
    """A generator that goes through all the files in the given directory that matches
    at least one of the patterns.

    Patterns can either be strings used for globbing or filter functions that return
    True if the file needs to be considered."""
    for root, dirs, files in os.walk(directory):
        for file in files:
            filename = os.path.join(root, file)
            if matchFile(filename, validFiles):
                yield filename

        if recursive is False:
            break


class CurlDownloader:
    def __init__(self):
        self.contents = []
        self.c = pycurl.Curl()

    def callback(self, buf):
        self.contents.append(buf)

    def get(self, url):
        self.contents = []
        c = self.c
        c.setopt(c.URL, url)
        c.setopt(c.WRITEFUNCTION, self.callback)
        c.setopt(c.COOKIEFILE, '')
        c.setopt(c.FOLLOWLOCATION, 1)
        c.perform()

        self.contents = ''.join(self.contents)

        return self.contents

    def __del__(self):
        self.c.close()

def curlget(url):
    c = CurlDownloader()
    return c.get(url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import cPickle
import logging

log = logging.getLogger('smewt.base.cache')

globalCache = {}

def clear():
    log.info('Cache: clearing memory cache')
    global globalCache
    globalCache = {}

def load(filename):
    log.info('Cache: loading cache from %s' % filename)
    global globalCache
    try:
        globalCache = cPickle.load(open(filename, 'rb'))
    except IOError:
        log.warning('Cache: Cache file doesn\'t exist')
    except EOFError:
        log.error('Cache: cache file is corrupted... Please remove it.')

def save(filename):
    log.info('Cache: saving cache to %s' % filename)
    cPickle.dump(globalCache, open(filename, 'wb'))


def cachedmethod(function):
    '''Makes a method use the cache. WARNING: this can NOT be used with static functions'''

    def cached(*args):
        # removed the first element of args for the key, which is the instance pointer
        # we don't want the cache to know which instance called it, it is shared among all
        # instances of the same class
        fkey = str(args[0].__class__), function.__name__
        key = (fkey, args[1:])
        if key in globalCache:
            return globalCache[key]

        result = function(*args)

        globalCache[key] = result

        return result

    cached.__doc__ = function.__doc__
    cached.__name__ = function.__name__

    return cached



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import GraphAction, Media, Metadata, SmewtException
from smewt.media import Movie
from smewt.guessers.guesser import Guesser
from smewt.base import utils, textutils
import re
import logging

log = logging.getLogger('smewt.guessers.moviefilename')


def validYear(year):
    try:
        return int(year) > 1920 and int(year) < 2015
    except ValueError:
        return False

def guessXCT(filename):
    if not '[XCT]' in filename:
        return filename, {}

    filename = filename.replace('[XCT]', '')
    md = {}

    try:
        # find metadata
        mdstr = textutils.matchRegexp(filename, '\[(?P<mdstr>.*?)\]')['mdstr']
        filename = filename.replace(mdstr, '')

        # find subs
        subs = textutils.matchRegexp(mdstr, 'St[{\(](?P<subs>.*?)[}\)]')['subs']
        mdstr.replace(subs, '')
        md['subs'] = subs.split('-')

        # find audio
        audio = textutils.matchRegexp(mdstr, 'aac[0-9\.-]*[{\(](?P<audio>.*?)[}\)]')['audio']
        mdstr.replace(audio, '')
        md['language'] = audio.split('-')

        # find year: if we found it, then the english title of the movie is either what's inside
        # the parentheses before the year, or everything before the year
        title = filename
        years = [ m['year'] for m in textutils.multipleMatchRegexp(filename, '(?P<year>[0-9]{4})') if validYear(m['year']) ]
        if len(years) == 1:
            title = filename[:filename.index(years[0])]
        elif len(years) >= 2:
            log.warning('Ambiguous filename: possible years are ' + ', '.join(years))

        try:
            title = textutils.matchRegexp(title, '\((?P<title>.*?)\)')['title']
        except:
            pass

        md['title'] = title

    finally:
        return filename, md


'''
def VideoFilename(filename):
    parts = textutils.cleanString(filename).split()

    found = {} # dictionary of identified named properties to their index in the parts list

    # heuristic 1: find VO, sub FR, etc...
    for i, part in enumerate(parts):
        if matchRegexp(part, [ 'VO', 'VF' ]):
            found = { ('audio', 'VO'): i }

    # heuristic 2: match video size
    #rexp('...x...') with x > 200  # eg: (720, 480) -> property format = 16/9, etc...

    # we consider the name to be what's left at the beginning, before any other identified part
    # (other possibility: look at the remaining zones in parts which are not covered by any identified props, look for the first one, or the biggest one)
    name = ' '.join(parts[:min(found.values())])
'''

def cleanMovieFilename(filename):
    import os.path
    filename = os.path.basename(filename)
    md = {}

    # TODO: fix those cases

    # first apply specific methods which are very strict but have a very high confidence
    filename, md = guessXCT(filename)

    # DVDRip.Xvid-$(grpname)
    grpnames = [ '\.Xvid-(?P<releaseGroup>.*?)\.',
                 '\.DivX-(?P<releaseGroup>.*?)\.'
                 ]
    editions = [ '(?P<edition>(special|unrated|criterion).edition)'
                 ]
    audio = [ '(?P<audioChannels>5\.1)' ]

    specific = grpnames + editions + audio
    for match in textutils.matchAllRegexp(filename, specific):
        for key, value in match.items():
            md[key] = value
            filename = filename.replace(value, '')


    # remove punctuation for looser matching now
    seps = [ ' ', '-', '.', '_' ]
    for sep in seps:
        filename = filename.replace(sep, ' ')

    # TODO: replace this with a getMetadataGroups function that splits on parentheses/braces/brackets
    remove = [ '[', ']', '(', ')', '{', '}' ]
    for rem in remove:
        filename = filename.replace(rem, ' ')

    name = filename.split(' ')


    properties = { 'format': [ 'DVDRip', 'HDDVD', 'HDDVDRip', 'BDRip', 'R5', 'HDRip', 'DVD', 'Rip' ],
                   'container': [ 'avi', 'mkv', 'ogv', 'wmv', 'mp4', 'mov' ],
                   'screenSize': [ '720p' ],
                   'videoCodec': [ 'XviD', 'DivX', 'x264', 'Rv10' ],
                   'audioCodec': [ 'AC3', 'DTS', 'He-AAC', 'AAC-He', 'AAC' ],
                   'language': [ 'english', 'eng',
                                 'spanish', 'esp',
                                 'french', 'fr',
                                 'italian', # no 'it', too common a word in english
                                 'vo', 'vf'
                                 ],
                   'releaseGroup': [ 'ESiR', 'WAF', 'SEPTiC', '[XCT]', 'iNT', 'PUKKA', 'CHD', 'ViTE', 'DiAMOND', 'TLF',
                                     'DEiTY', 'FLAiTE', 'MDX', 'GM4F', 'DVL', 'SVD', 'iLUMiNADOS', ' FiNaLe', 'UnSeeN' ],
                   'other': [ '5ch', 'PROPER', 'REPACK', 'LIMITED', 'DualAudio', 'iNTERNAL', 'Audiofixed',
                              'classic', # not so sure about this one, could appear in a title
                              'ws', # widescreen
                              'SE', # special edition
                              # TODO: director's cut
                              ],
                   }

    # ensure they're all lowercase
    for prop, value in properties.items():
        properties[prop] = [ s.lower() for s in value ]


    # to try to guess what part of the filename is the movie title, we only keep as
    # possible title the first characters of the filename up to the leftmost metadata
    # element we found, no more
    minIdx = len(name)

    # get specific properties
    for prop, value in properties.items():
        for part in name:
            if part.lower() in value:
                md[prop] = part
                minIdx = min(minIdx, name.index(part))


    # get year
    def validYear(year):
        try:
            return int(year) > 1920 and int(year) < 2015
        except ValueError:
            return False


    for part in name:
        year = textutils.stripBrackets(part)
        if validYear(year):
            md['year'] = int(year)
            minIdx = min(minIdx, name.index(part))

    # remove ripper name
    for by, who in zip(name[:-1], name[1:]):
        if by.lower() == 'by':
            md['ripper'] = who
            minIdx = min(minIdx, name.index(by))

    # subtitles
    for sub, lang in zip(name[:-1], name[1:]):
        if sub.lower() == 'sub':
            md['subtitleLanguage'] = lang
            minIdx = min(minIdx, name.index(sub))

    # get CD number (if any)
    cdrexp = re.compile('[Cc][Dd]([0-9]+)')
    for part in name:
        try:
            md['cdNumber'] = int(cdrexp.search(part).groups()[0])
            minIdx = min(minIdx, name.index(part))
        except AttributeError:
            pass

    name = ' '.join(name[:minIdx])
    minIdx = len(name)

    # last chance on the full name: try some popular regexps
    general = [ '(?P<dircut>director\'s cut)',
                '(?P<edition>edition collector)' ]
    websites = [ 'sharethefiles.com' ]
    websites = [ '(?P<website>%s)' % w.replace('.', ' ') for w in websites ] # dots have been previously converted to spaces
    rexps = general + websites

    matched = textutils.matchAllRegexp(name, rexps)
    for match in matched:
        for key, value in match.items():
            minIdx = min(minIdx, name.find(value))

    name = name[:minIdx]

    # try website names
    # TODO: generic website url guesser
    websites = [ 'sharethefiles.com' ]


    # return final name as a guess for the movie title
    md['title'] = name
    return md


class MovieFilename(GraphAction):

    supportedTypes = [ 'video', 'subtitle' ]

    def __init__(self):
        super(MovieFilename, self).__init__()

    def canHandle(self, query):
        media = query.find_one(Media)
        if media.type() not in ('video', 'subtitle'):
            raise SmewtException("%s: can only handle video or subtitle media objects: %s" % (self.__class__.__name__, media.filename))

    def perform(self, query):
        self.checkValid(query)
        media = query.find_one(node_type = Media)

        movie = cleanMovieFilename(media.filename)

        #log.info('Found filename information from %s: %s' % (media.filename, movie))

        media.matches = query.Movie(**movie)
        return query



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewtexception import SmewtException
from urlparse import ParseResult, urlparse, urlunparse
from urllib import urlencode, unquote_plus
import textutils

class SmewtUrl:
    def __init__(self, type=None, path=None, args = {}, url = None):
        if type and path and not url:
            if path[0] != '/': path = '/' + path
            args = textutils.toUtf8(args)
            self.spath = ParseResult('http', type, path, '', urlencode(args), None)

        elif url and not type and not path:

            if not unicode(url).startswith('smewt://'):
                raise SmewtException('Could not create SmewtUrl from %s' % url)

            url = str(url).replace('smewt://', 'http://')
            self.spath = urlparse(url)
        else:
            raise SmewtException('SmewtUrl: you need to specify either a string url or the components of the SmewtUrl you want to build')

        # set member vars in function of url type
        self.mediaType = self.viewType = self.actionType = None

        try:
            if self.spath.netloc == 'media':
                self.mediaType, self.viewType = self.spath.path.split('/')[1:]
            elif self.spath.netloc == 'action':
                self.actionType, = self.spath.path.split('/')[1:]
            elif self.spath.netloc == 'feedwatcher':
                pass
            else:
                raise SmewtException("SmewtUrl: invalid url type '%s'" % self.spath.netloc)

        except ValueError:
            raise SmewtException("SmewtUrl: incomplete url '%s'" % self)

        # TODO: in python 2.6 use parse_qs
        if self.spath.query:
            self.args = dict([ kv.split('=') for kv in self.spath.query.split('&') ])
            for key, value in self.args.items():
                self.args[unquote_plus(key)] = unquote_plus(value).decode('utf-8')

    def __str__(self):
        return urlunparse(self.spath).replace('http://', 'smewt://')

    def __unicode__(self):
        return urlunparse(self.spath).replace('http://', 'smewt://')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime
from PyQt4.QtCore import Qt, QVariant, QAbstractListModel

class Event:
    def __init__(self, text):
        self.timestamp = datetime.datetime.now()
        self.text = text

    def __str__(self):
        return self.timestamp.ctime() + ': ' + self.text

class EventList(QAbstractListModel):
    def __init__(self):
        super(EventList, self).__init__()
        self.events = []

    def rowCount(self, parent):
        return len(self.events)

    def data(self, index, role = Qt.DisplayRole):
        if role != Qt.DisplayRole:
            return QVariant()

        return QVariant(str(self.events[index.row()]))

    def add(self, text):
        self.events.append(Event(text))
        # TODO: use dataChanged instead
        self.reset()


class EventServer:

    events = EventList()

    @staticmethod
    def publish(text):
        EventServer.events.add(text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import utils
from dirselector import *
from PyQt4.QtCore import *
from PyQt4.QtGui import *
import os
import sys
import logging

log = logging.getLogger('smewt.gui.collectionsfolderpage')

DEFAULT_WIDTH, DEFAULT_HEIGHT = 500, 600

class CollectionFoldersPage(QDialog):
    def __init__(self,
                 parent,
                 description,
                 collection,
                 #settingKeyFolders = 'collection_folders',
                 #settingKeyRecursive = 'collection_folders_recursive',
                 ):

        QDialog.__init__(self, parent)

        self.collection = collection

        #    if sys.platform == 'darwin' and not self.folders:
        #        self.folders = [ os.environ['HOME'] + '/Movies' ]

        # remove directories which don't exist to avoid a segfault later
        self.folders = [ folder for folder in collection.folders if os.path.isdir(folder) ]

        try:
            self.recursiveSelection = collection.folders.items()[0]
        except:
            self.recursiveSelection = True

        self.layout = QVBoxLayout()

        self.instructions_label = QLabel(description)
        self.layout.addWidget(self.instructions_label)
        self.dirselector = DirSelector(folders = self.folders, recursiveSelection = self.recursiveSelection)

        log.debug('Checked folders: %s' % self.folders)
        if not self.folders:
            if sys.platform in ('darwin', 'linux2'):
                log.debug('Expanding folder: %s' % os.environ['HOME'])
                self.dirselector.expandPathNode(os.environ['HOME'])

        # if we have any folders in our list, try to expand the DirSelector in a smart way
        elif len(self.folders) == 1:
            dir = utils.parentDirectory(self.folders[0])
            log.info('Expanding folder: %s' % dir)
            self.dirselector.expandPathNode(dir)

        else:
            dir = utils.commonRoot(self.folders)
            log.debug('Expanding folder: %s' % dir)
            self.dirselector.expandPathNode(dir)


        self.connect(self.dirselector, SIGNAL('selectionChanged'), self.selectionChanged)
        self.layout.addWidget(self.dirselector)

        self.button_layout = QHBoxLayout()

        self.ok_button = QPushButton("Ok")
        self.apply_button = QPushButton("Apply")
        self.apply_button.setEnabled(False)
        self.cancel_button = QPushButton("Cancel")

        self.button_layout.addStretch()
        self.button_layout.addWidget(self.ok_button)
        self.button_layout.addWidget(self.apply_button)
        self.button_layout.addWidget(self.cancel_button)

        self.layout.addLayout(self.button_layout)

        self.setLayout(self.layout)

        self.connect(self.ok_button, SIGNAL('clicked()'), self.ok)
        self.connect(self.apply_button, SIGNAL('clicked()'), self.apply)
        self.connect(self.cancel_button, SIGNAL('clicked()'), self.cancel)

        size = QSize(DEFAULT_WIDTH, DEFAULT_HEIGHT)
        self.resize(size)

    def getFolders(self):
        folders = [ str(f) for f in self.dirselector.selectedFolders() ]
        recursiveSelection = self.dirselector.recursiveSelection()

        return dict((folder, recursiveSelection) for folder in folders)

    def ok(self):
        self.apply()
        self.done(1)

    def apply(self):
        self.collection.setFolders(self.getFolders())

        self.apply_button.setEnabled(False)

    def cancel(self):
        self.done(0)

    def selectionChanged(self):
        originalFolders = set([os.path.abspath(f) for f in self.folders])
        newFolders = set([os.path.abspath(str(f)) for f in self.dirselector.selectedFolders()])
        if originalFolders != newFolders or self.recursiveSelection != self.dirselector.recursiveSelection():
            self.apply_button.setEnabled(True)

        else:
            self.apply_button.setEnabled(False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import with_statement
from smewtexception import SmewtException
from Queue import PriorityQueue
from threading import Thread, Lock, current_thread
from PyQt4 import QtCore
import time
import logging

log = logging.getLogger('smewt.base.taskmanager')

class Task(object):
    def __init__(self, priority = 5):
        self.priority = priority

    def perform(self):
        """All tasks should implement this function, which should perform the actual task.

        No data is passed to this method. All the necessary task data should be given in the constructor.

        This method shouldn't return anything on success, and raise an exception in case of failure.
        It can have side effects though, such as updating the global collection for instance. In this case,
        the collection needs to be passed as argument to the constructor."""
        raise NotImplementedError


def worker(taskManager):
    log.info('Worker thread is: %d' % current_thread().ident)
    while True:
        if taskManager.shouldFinish:
            log.info('Worker thread stopped working because TaskManager should finish now')
            return

        try:
            taskId = taskManager.taskId
            (_, _), task = taskManager.queue.get()
            taskManager.taskId + 1

            # TODO: need to have timeout on the tasks, eg: 5 seconds
            # TODO: need to be able to stop task immediately
            task.perform()

        except Exception, e:
            import sys, traceback
            log.warning('TaskManager: task failed with error: %s' % ''.join(traceback.format_exception(*sys.exc_info())))

        finally:
            taskManager.taskDone(taskId)


class TaskManager(QtCore.QObject):
    """The TaskManager is a stable priority queue of tasks. It takes them one by one, the
    one with the highest priority first, and call its perform() method, then repeat until no tasks are left.

    If two or more tasks have the same priority, it will take the one that was added first to the queue.

    The TaskManager can be controlled asynchronously, as it runs the tasks in a separate thread."""

    progressChanged = QtCore.pyqtSignal(int, int)

    def __init__(self):
        super(TaskManager, self).__init__()

        # our main task queue
        self.queue = PriorityQueue()

        self.taskId = 0    # ID for the next task that will be generated
        self.total = 0     # used to keep track of the total jobs that have been submitted (queue size decreases as we process tasks)
        self.finished = [] # list of task IDs which have finished

        self.lock = Lock()

        log.info('Main GUI thread is: %d' % current_thread().ident)

        self.shouldFinish = False

        # ideally a pool of threads or sth like TBB or ThreadWeaver
        self.workerThread = Thread(target = worker, args = (self,))
        self.workerThread.daemon = True
        self.workerThread.start()


    def callback(self):
        # we need to send a signal here so that it can be dealt with properly by the gui thread
        # (we don't want to execute a gui callback using a worker thread)
        self.progressChanged.emit(len(self.finished), self.total)


    def add(self, task):
        log.debug('TaskManager add task')
        with self.lock:
            # -task.priority because it always gets the lowest one first
            # we need to put the time as well, because Queue uses heap sort which is not stable, so we
            # had to find a way to make it look stable ;-)
            self.queue.put(( (-task.priority, time.time()), task ))
            self.total += 1

            self.callback()


    def taskDone(self, taskId):
        log.debug('TaskManager task done')
        with self.lock:
            self.finished.append(taskId)

            log.info('Task %d/%d completed!' % (len(self.finished), self.total))

            # if we finished all the tasks, reset the current total
            if self.queue.empty():
                self.finished = []
                self.total = 0

            self.queue.task_done()
            self.callback()

    def finishNow(self):
        log.info('TaskManager should finish ASAP, waiting for currently running tasks to finish')
        self.shouldFinish = True
        with self.lock:
            if self.total == 0:
                # worker thread is already waiting on an empty queue, we can't wait for him
                log.info('No currently running jobs')
                return

        # FIXME: need to stop worker, we can't always wait for him
        self.workerThread.join()

        log.info('TaskManager: last running task finished')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import GraphAction, cachedmethod, utils, textutils, SmewtException, Media
from smewt.base.utils import smewtDirectory
from smewt.base.textutils import toUtf8
from smewt.base.mediaobject import foundMetadata
from smewt.guessers.guesser import Guesser
from smewt.media import Movie
from pygoo import MemoryObjectGraph
from urllib import urlopen,  urlencode
from tvdbmetadataprovider import TVDBMetadataProvider
import logging

log = logging.getLogger('smewt.guessers.movietmdb')

class MovieTMDB(GraphAction):

    supportedTypes = [ 'video', 'subtitle' ]

    def canHandle(self, query):
        return True

    def perform(self, query):
        self.checkValid(query)

        log.debug('MovieTvdb: finding more info on %s' % query.find_one(Movie))
        movie = query.find_one(Movie)

        try:
            mdprovider = TVDBMetadataProvider()
            result = mdprovider.startMovie(movie.title)
        except SmewtException, e:
            # movie could not be found, return a dummy Unknown movie instead so we can group them somewhere
            log.warning('Could not find info for movie: %s' % toUtf8(query.find_one(Media).filename))
            noposter = smewtDirectory('smewt', 'media', 'common', 'images', 'noposter.png')
            result = MemoryObjectGraph()
            result.Movie(title = 'Unknown', loresImage = noposter, hiresImage = noposter)

        result = foundMetadata(query, result.find_one(Movie))
        return result




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import SIGNAL, QObject
from smewtexception import SmewtException
from mediaobject import Media, Metadata
from subtitletask import SubtitleTask
from smewt.media import Series
import sys, time, logging


log = logging.getLogger('smewt.base.actionfactory')

# Singleton pattern found at http://code.activestate.com/recipes/66531/
class Singleton(object):
    def __new__(cls):
        if not '_the_instance' in cls.__dict__:
            cls._the_instance = object.__new__(cls)
        return cls._the_instance


# TODO: actions in the ActionFactory should be registered as plugins that
# can provide certain types of service, ie: getsubtitles action may be filled
# by tvsubtitles, opensubtitles, etc...
class ActionFactory(Singleton):
    def __init__(self):
        self.subtitleProviders = []
        self.registerPlugins()

    def registerPlugins(self):
        # subs += opensubtitles
        from smewt.media.subtitle.subtitle_tvsubtitles_provider import TVSubtitlesProvider
        self.subtitleProviders.append(TVSubtitlesProvider())

    def dispatch(self, mainWidget, surl):
        if surl.actionType == 'play':
            if sys.platform == 'linux2':
                action = 'smplayer'
                args = [ '-fullscreen', '-close-at-end' ]

                nfile = 1
                while 'filename%d' % nfile in surl.args:
                    filename = surl.args['filename%d' % nfile]
                    args.append(filename)

                    # update last viewed info
                    media = mainWidget.smewtd.database.find_one(Media, filename = filename)
                    media.metadata.lastViewed = time.time()

                    if 'subtitle%d' % nfile in surl.args:
                        args += [ '-sub', surl.args['subtitle%d' % nfile] ]

                    nfile += 1

            elif sys.platform == 'darwin':
                action = 'open'
                args = []
                nfile = 1
                while 'filename%d' % nfile in surl.args:
                    filename = surl.args['filename%d' % nfile]
                    args.append(filename)
                    nfile += 1

            log.debug('launching %s with args = %s' % (action, str(args)))
            mainWidget.externalProcess.startDetached(action, args)

        elif surl.actionType == 'getsubtitles':
            title = surl.args['title']
            language = surl.args['language']

            for provider in self.subtitleProviders:
                subTask = SubtitleTask(mainWidget.smewtd.database, provider, title, language)
                mainWidget.smewtd.taskManager.add(subTask)

        else:
            raise SmewtException('Unknown action type: %s' % surl.actionType)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import cachedmethod, utils, SmewtException, Media
from smewt.guessers.guesser import Guesser
from smewt.media import Episode, Series, Movie
from smewt.base import textutils
from smewt.base.utils import smewtDirectory, smewtUserDirectory
from pygoo import MemoryObjectGraph

from PyQt4.QtCore import SIGNAL, QObject, QUrl

import os, sys, re, logging
from urllib import urlopen,  urlencode
from smewt.base.utils import curlget

import thetvdbapi
import tmdb

import datetime

log = logging.getLogger('smewt.guessers.tvdbmetadataprovider')

class TVDBMetadataProvider(object):
    def __init__(self):
        super(TVDBMetadataProvider, self).__init__()

        self.tvdb = thetvdbapi.TheTVDB("65D91F0290476F3E")
        self.tmdb = tmdb.MovieDb()

    @cachedmethod
    def getSeries(self, name):
        """Get the TVDBPy series object given its name."""
        results = self.tvdb.get_matching_shows(name)
        for id, name in results:
            # FIXME: that doesn't look correct: either yield or no for
            return id
        raise SmewtException("EpisodeTVDB: Could not find series '%s'" % name)


    @cachedmethod
    def getEpisodes(self, series):
        """From a given TVDBPy series object, return a graph containing its information
        as well as its episodes nodes."""
        show, episodes = self.tvdb.get_show_and_episodes(series)

        # TODO: debug to see if this is the correct way to access the series' title
        result = MemoryObjectGraph()
        smewtSeries = result.Series(title = show.name)

        for episode in episodes:
            ep = result.Episode(series = smewtSeries,
                                season = episode.season_number,
                                episodeNumber = episode.episode_number)
            ep.set('title', episode.name)
            ep.set('synopsis', episode.overview)
            ep.set('originalAirDate', str(episode.first_aired))

        return result

    @cachedmethod
    def getMovie(self, name):
        """Get the IMDBPy movie object given its name."""
        if not name:
            raise SmewtException('You need to specify at least a probable name for the movie...')
        log.debug('MovieTMDB: looking for movie %s', name)
        results = self.tmdb.search(name)
        for r in results:
          return r['id']

        raise SmewtException("MovieTMDB: Could not find movie '%s'" % name)

    @cachedmethod
    def getMovieData(self, movieId):
        """From a given TVDBPy movie object, return a graph containing its information."""
        m = self.tmdb.getMovieInfo(movieId)

        result = MemoryObjectGraph()
        movie = result.Movie(title = unicode(m['name']))

        if m.get('released'):
            movie.set('year', datetime.datetime.strptime(m['released'], '%Y-%m-%d').year)

        movie.set('director', [unicode(d['name']) for d in m['cast'].get('director', [])])
        movie.set('writer', [unicode(d['name']) for d in m['cast'].get('author', [{'name': ''}])])
        movie.set('genres', [unicode(g) for g in m['categories'].get('genre', {}).keys()])
        movie.set('rating', m['rating'])
        movie.set('plot', [unicode(m['overview'])])

        try:
            movie.cast = [ unicode(actor['name']) + ' -- ' + unicode(actor['character']) for actor in m['cast']['actor'][:15] ]
        except KeyError:
            movie.cast = []


        return result


    @cachedmethod
    def getSeriesPoster(self, tvdbID):
        """Return the low- and high-resolution posters (if available) of an tvdb object."""
        imageDir = smewtUserDirectory('images')
        noposter = smewtDirectory('smewt', 'media', 'common', 'images', 'noposter.png')

        loresFilename, hiresFilename = None, None

        urls = self.tvdb.get_show_image_choices(tvdbID)
        posters = [url for url in urls if url[1]=='poster']
        if len(posters)>0:
            loresURL = posters[0][0]
            loresFilename = os.path.join(imageDir, '%s_lores.jpg' % tvdbID)
            open(loresFilename, 'wb').write(curlget(loresURL))

            if len(posters)>1:
              hiresURL = posters[1][0]
              hiresFilename = os.path.join(imageDir, '%s_hires.jpg' % tvdbID)
              open(hiresFilename, 'wb').write(curlget(hiresURL))
        else:
            log.warning('Could not find poster for tvdb ID %s' % tvdbID)
            return (noposter, noposter)

        return (loresFilename, hiresFilename)


    @cachedmethod
    def getMoviePoster(self, movieId):
        """Return the low- and high-resolution posters (if available) of an tvdb object."""
        imageDir = smewtUserDirectory('images')
        noposter = smewtDirectory('smewt', 'media', 'common', 'images', 'noposter.png')

        loresFilename, hiresFilename = None, None

        m = self.tmdb.getMovieInfo(movieId)

        posters = []
        for poster in m['images'].posters:
            for key, value in poster.items():
                if key not in ['id', 'type']:
                    if value.startswith("http://"):
                      posters.append(value)


        if len(posters)>0:
            loresURL = posters[0]
            loresFilename = os.path.join(imageDir, '%s_lores.jpg' % movieId)
            open(loresFilename, 'wb').write(curlget(loresURL))

            if len(posters)>1:
              hiresURL = posters[1]
              hiresFilename = os.path.join(imageDir, '%s_hires.jpg' % movieId)
              open(hiresFilename, 'wb').write(curlget(hiresURL))
        else:
            log.warning('Could not find poster for tmdb ID %s' % movieId)
            return (noposter, noposter)

        return (loresFilename, hiresFilename)


    def startEpisode(self, episode):
        if episode.get('series') is None:
            raise SmewtException("TVDBMetadataProvider: Episode doesn't contain 'series' field: %s", md)

        name = episode.series.title
        series = self.getSeries(name)
        eps = self.getEpisodes(series)

        try:
            lores, hires = self.getSeriesPoster(series)
            eps.find_one(Series).update({ 'loresImage': lores,
                                          'hiresImage': hires })
            return eps

        except Exception, e:
            log.warning(str(e) + ' -- ' + str(textutils.toUtf8(episode)))
            return MemoryObjectGraph()

    def startMovie(self, movieName):
        try:
            movieTvdb = self.getMovie(movieName)
            result = self.getMovieData(movieTvdb)

            movie = result.find_one('Movie')
            lores, hires = self.getMoviePoster(movieTvdb)
            movie.loresImage = lores
            movie.hiresImage = hires

            #result.display_graph()
            return result

        except SmewtException, e:
            raise




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt import SmewtException, SmewtUrl, EventServer
from PyQt4.QtCore import SIGNAL, QVariant, QProcess, QThread, QTimer, QString
from PyQt4.QtGui import QWidget, QPushButton, QHBoxLayout, QVBoxLayout, QFileDialog, QListWidget, QListView, QInputDialog, QLineEdit, QAbstractItemView, QLabel, QMessageBox, QDialog, QListWidget
from smewt.plugins.amulefeedwatcher import AmuleFeedWatcher


class CheckThread(QThread):
    def __init__(self, parent, feedList):
        super(CheckThread, self).__init__(parent)
        self.feedList = feedList

    def run(self):
        self.feedList.checkAllFeeds()

class EpisodeSelector(QDialog):
    def __init__(self, feed):
        super(EpisodeSelector, self).__init__()

        layout = QVBoxLayout()
        layout.addWidget(QLabel(feed['title']))

        self.eplist = QListWidget()
        self.eps = sorted(feed['entries'], key = lambda x: x['updated'])
        selectionIndex = -1
        i = 0
        for ep in self.eps:
            self.eplist.addItem(QString(ep['title']))
            if ep['updated'] == feed['lastUpdate']:
                selectionIndex = i
            i += 1
        self.eplist.setCurrentRow(selectionIndex)

        layout.addWidget(self.eplist)

        cancelButton = QPushButton('Cancel')
        okButton = QPushButton('Set as last episode')
        self.connect(cancelButton, SIGNAL('clicked()'),
                     self.reject)
        self.connect(okButton, SIGNAL('clicked()'),
                     self.acceptLocal)

        buttonsLayout = QHBoxLayout()
        buttonsLayout.addStretch()
        buttonsLayout.addWidget(cancelButton)
        buttonsLayout.addWidget(okButton)

        layout.addLayout(buttonsLayout)

        self.setLayout(layout)

    def acceptLocal(self):
        self.lastUpdate = self.eps[self.eplist.currentRow()]['updated']
        self.accept()



class FeedWatchWidget(QWidget):
    def __init__(self):
        super(FeedWatchWidget, self).__init__()

        self.feedList = AmuleFeedWatcher()
        self.eventList = EventServer.events

        # Control buttons: check now, add feed, remove feed
        checkNowButton = QPushButton('Check feeds now!')
        self.connect(checkNowButton, SIGNAL('clicked()'),
                     self.checkNow)

        addFeedButton = QPushButton('Add feed...')
        self.connect(addFeedButton, SIGNAL('clicked()'),
                     self.addFeed)

        removeFeedButton = QPushButton('Remove feed...')
        self.connect(removeFeedButton, SIGNAL('clicked()'),
                     self.removeFeed)

        buttons = QHBoxLayout()
        buttons.addWidget(addFeedButton)
        buttons.addWidget(removeFeedButton)
        buttons.addStretch()
        buttons.addWidget(checkNowButton)

        # main layout
        layout = QVBoxLayout()
        self.feedView = QListView()
        self.feedView.setModel(self.feedList)
        layout.addWidget(self.feedView)

        self.connect(self.feedView, SIGNAL('doubleClicked(const QModelIndex&)'),
                     self.feedEdit)

        layout.addLayout(buttons)

        self.eventView = QListView()
        self.eventView.setSelectionMode(QAbstractItemView.NoSelection)
        self.eventView.setModel(self.eventList)
        layout.addWidget(self.eventView)

        self.setLayout(layout)

        # check for feeds every 2 hours
        self.timer = QTimer(self)
        self.connect(self.timer, SIGNAL('timeout()'),
                     self.checkNow)
        self.timer.start(2*60*60*1000)

    def feedEdit(self, index):
        fullFeed = self.feedList.getFullFeedIndex(index.row())
        epSelect = EpisodeSelector(fullFeed)
        if epSelect.exec_() == QDialog.Accepted:
            self.feedList.setLastUpdateIndex(index.row(), epSelect.lastUpdate)

    def checkNow(self):
        # this can last a while, do not block the UI meanwhile
        CheckThread(self, self.feedList).start()

    def addFeed(self):
        result, ok = QInputDialog.getText(self, 'Add feed...', 'Enter the feed url:', QLineEdit.Normal, '')
        if ok and not result.isEmpty():
            try:
                self.feedList.addFeed(result)
            except Exception, e:
                QMessageBox.critical(self, 'Add feed...', 'Error: %s' % str(e))

    def removeFeed(self):
        idx = self.feedView.currentIndex().row()
        if idx >= 0:
            self.feedList.removeFeedIndex(idx)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2010 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import QSettings, QVariant
from pygoo import MemoryObjectGraph, Equal
import smewt
from smewt.media import Series, Episode, Movie
from smewt.base import utils, Collection, Media
from smewt.base.taskmanager import Task, TaskManager
from os.path import join, dirname, splitext, getsize
from smewt.taggers import EpisodeTagger, MovieTagger
import time, logging

log = logging.getLogger('smewt.base.smewtdaemon')


class VersionedMediaGraph(MemoryObjectGraph):

    def add_object(self, node, recurse = Equal.OnIdentity, excluded_deps = list()):
        result = super(VersionedMediaGraph, self).add_object(node, recurse, excluded_deps)
        if isinstance(result, Media):
            result.lastModified = time.time()

        return result


def validEpisode(filename):
    return utils.matchFile(filename, [ '*.avi', '*.ogm', '*.mkv' ]) #and getsize(filename) < 600 * 1024 * 1024

def validMovie(filename):
    return utils.matchFile(filename, [ '*.avi', '*.ogm', '*.mkv' ]) #and getsize(filename) > 600 * 1024 * 1024


class SmewtDaemon(object):
    def __init__(self, progressCallback = None):
        super(SmewtDaemon, self).__init__()

        # get a TaskManager for all the import tasks
        self.taskManager = TaskManager()
        self.taskManager.progressChanged.connect(self.progressChanged)

        # get our main graph DB
        self.loadDB()

        # get our collections: series and movies for now
        self.episodeCollection = Collection(name = 'Series',
                                           # episodes are videos < 600MB and/or subtitles
                                           validFiles = [ validEpisode, '*.idx', '*.sub', '*.srt' ],
                                           mediaTagger = EpisodeTagger,
                                           dataGraph = self.database,
                                           taskManager = self.taskManager)


        self.movieCollection = Collection(name = 'Movie',
                                          # movies are videos > 600MB and/or subtitles
                                          validFiles = [ validMovie, '*.idx', '*.sub', '*.srt' ],
                                          mediaTagger = MovieTagger,
                                          dataGraph = self.database,
                                          taskManager = self.taskManager)



    def progressChanged(self, current, total):
        if total == 0:
            self.saveDB()


    def loadDB(self):
        log.info('Loading database...')
        settings = QSettings()
        dbfile = unicode(settings.value('database_file').toString())
        if not dbfile:
            dbfile = join(utils.smewtUserDirectory(), smewt.APP_NAME + '.database')
            settings.setValue('database_file', QVariant(dbfile))

        self.database = VersionedMediaGraph()
        try:
            self.database.load(dbfile)
        except:
            log.warning('Could not load database %s', dbfile)

    def saveDB(self):
        log.info('Saving database...')
        self.database.save(unicode(QSettings().value('database_file').toString()))

    def quit(self):
        log.info('SmewtDaemon quitting...')
        self.taskManager.finishNow()
        self.saveDB()
        log.info('SmewtDaemon quitting OK!')

    def updateCollections(self):
        self.episodeCollection.update()
        self.movieCollection.update()

    def rescanCollections(self):
        self.episodeCollection.rescan()
        self.movieCollection.rescan()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import Media, Metadata
from smewt.solvers.solver import Solver
from smewt.base.textutils import levenshtein
import logging

log = logging.getLogger('smewt.solvers.simplesolver')


def exactMatch(baseGuess, md):
    return baseGuess.unique_key() == md.unique_key()

def fuzzyMatch(baseGuess, md):
    for p1, p2 in zip(baseGuess.unique_key(), md.unique_key()):
        if isinstance(p1, basestring):
            if p1 not in p2:
                return False
        else:
            if p1 != p2:
                return False
    return True

def fuzzyMatch2(baseGuess, md):
    for p1, p2 in zip(baseGuess.unique_key(), md.unique_key()):
        if type(p1) == str or type(p1) == unicode:
            # TODO: levenshtein doesn't cut it here, we need a better string distance
            if levenshtein(p1.lower(), p2.lower()) > 80:
                return False
        elif isinstance(p1, Metadata):
            if not fuzzyMatch2(p1, p2):
                return False
        else:
            if p1 != p2:
                return False
    return True


class SimpleSolver(Solver):
    '''This solver implements this simple solving strategy:
    - first find the metadata which represents a unique object with
      highest confidence. If not found, the solver failed.
    - then merges all the information there is in all the guesses which have the
      same unique ID.
    '''

    def __init__(self, type):
        super(SimpleSolver, self).__init__()
        self.type = type

    def perform(self, query):
        self.checkValid(query)

        #query.display_graph()

        metadata = query.find_all(node_type = self.type)

        # 1- get a node that looks like it could be our potential candidate
        baseGuess, confidence = None, -1
        for md in metadata:
            if md.is_unique() and md.get('confidence') > confidence:
                baseGuess, confidence = md, md.get('confidence')

        if baseGuess is None:
            raise SmewtException('SimpleSolver could not find base guess')

        if confidence < 0.9:
            log.warning('Base guess for %s looks shady, confidence = %f: %s' % (query.find_one(Media).filename, confidence, baseGuess))
            #return self.found(query, None)

        # 2- once we have it, merge data from other nodes that look like him
        for md in metadata:
            # do not inadvertently overwrite some data we could have found from another instance
            if md is baseGuess:
                continue

            # if there is a match, merge the data
            if fuzzyMatch2(baseGuess, md):
                baseGuess.update(dict(md.items()))

        return self.found(query, baseGuess)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from pygoo import BaseObject
from smewt.base import Metadata



class Movie(Metadata):

    typename = 'Movie'

    schema = { 'title': unicode,
               'year': int,
               # more to come
               }

    valid = [ 'title' ]

    unique = [ 'title', 'year' ]

    order = [ 'title', 'year' ]

    converters = {}


class Comment(BaseObject):
    schema = { 'metadata': Metadata,
               'author': unicode,
               'text': unicode,
               'date': int
               }

    reverse_lookup = { 'metadata': 'comments' }

    valid = [ 'metadata', 'author', 'date' ]
    unique = [ 'metadata', 'author', 'date' ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import SIGNAL, QObject
from smewt.base import SmewtException
from smewt.base.mediaobject import Media

class GraphAction(object):
    """A GraphAction operates a transformation of a graph containing one Media object.
    In particular, they will tend to be either a Guesser, which will try to guess more
    information from the given media object (either from the filename, or from the web, ...),
    or a Solver, which will take the information in the graph and try to sum it up and only
    keep the relevant one."""

    def __init__(self):
        super(GraphAction, self).__init__()


    def canHandle(self, media):
        """Return silently if this GraphAction can handle the given media object.
        Should raise an exception otherwise."""
        raise NotImplementedError
        #if media[0].type() not in self.supportedTypes:
        #    raise SmewtException('Guesser: this guesser only supports files of type: %s but you provided a file of type: %s' % (str(self.supportedTypes), media[0].type()))


    def checkValid(self, query):
        """Check that our query graph contains only one MEdia object and that it can be handled
        by our GraphAction."""
        media = query.find_all(Media)
        if len(media) != 1:
            raise SmewtException('%s: your query should contain exactly 1 element in the query graph' % self.__class__.__name__)

        self.canHandle(query)


    def perform(self, query):
        """This method should do the actual work and return the result directly.
        When this method is called, ownership of the query graph is yielded to this GraphAction,
        which means that it can safely tamper with it."""
        raise NotImplementedError


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import with_statement
from PyQt4.QtCore import SIGNAL, Qt, QObject
from pygoo import MemoryObjectGraph
from smewt.base import Media, Task, SmewtException, textutils
from smewt.base.utils import tolist
from smewt.media import Series, Subtitle, SubtitleNotFoundError
import os.path

import itertools
import logging

log = logging.getLogger('smewt.subtitletask')

# TODO: should be renamed SeriesSubtitleTask
class SubtitleTask(Task):
    def __init__(self, database, provider, title, language):
        super(SubtitleTask, self).__init__()
        self.database = database
        self.provider = provider
        self.series = database.find_one(Series, title = title)
        self.language = language

        log.info('Creating SubtitleTask for all files from series: %s' % self.series.title)


    def perform(self):
        languageMap = { 'en': u'English', 'fr': u'Fran\xe7ais', 'es': u'Espa\xf1ol' }

        # find objects which don't have yet a subtitle of the desired language
        seriesEpisodes = set(tolist(self.series.episodes))
        currentSubs = self.database.find_all(node_type = Subtitle,
                                             valid_node = lambda x: x.metadata in seriesEpisodes,
                                             language = self.language)

        alreadyGood = set([ s.metadata for s in currentSubs ])

        episodes = [ ep for ep in seriesEpisodes if ep not in alreadyGood ]
        if not episodes:
            log.info('All videos already have subtitles!')

        subs = MemoryObjectGraph()

        for ep in episodes:
            try:
                # the following assumes we always have a single metadata object for this file
                videoFilename = ep.files.filename
                subFilename = os.path.splitext(videoFilename)[0] + '.%s.srt' % languageMap[self.language]

                # if file doesn't exist yet, try to download it
                if not os.path.isfile(subFilename):
                    possibleSubs = self.provider.getAvailableSubtitles(ep).find_all(language = self.language)
                    if not possibleSubs:
                        raise SubtitleNotFoundError('didn\'t find any possible subs...')

                    # we could do a more elaborate selection here if more than 1 result is returned
                    candidate = possibleSubs[0]
                    if len(possibleSubs) > 1:
                        log.warning('More than 1 possible subtitle found: %s', str(possibleSubs))
                        dists = [ (textutils.levenshtein(videoFilename, sub.title), sub) for sub in possibleSubs ]
                        candidate = sorted(dists)[0][1]
                        log.warning('Choosing %s' % candidate)

                    log.info('Trying to download subs for %s' % ep.files.filename)
                    subtext = self.provider.getSubtitle(candidate)

                    # write the subtitle in the appropriate file
                    # (here as well we might want to look at how many results we have (ie: movie
                    # spanning multiple CDs, etc...)
                    with open(subFilename, 'w') as f:
                        f.write(subtext)
                    log.info('Found subtitle for %s' % ep.files.filename)

                else:
                    log.warning('\'%s\' already exists. Not overwriting it...' % subFilename)

                # update the found subs with this one
                sub = self.database.Subtitle(metadata = ep, language = self.language)
                subfile = self.database.Media(filename = subFilename, metadata = sub)

            except SubtitleNotFoundError, e:
                log.warning('subno: did not found any subtitle for %s: %s', str(video), str(e))

            except SmewtException, e:
                log.warning('se: did not found any subtitle for %s: %s' % (str(video), str(e)))


        log.debug('SubtitleTask: all done!')



"""
thetvdb.com Python API
(c) 2009 James Smith (http://loopj.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import urllib
import datetime
import re

import xml.etree.cElementTree as ET

class TheTVDB(object):
    def __init__(self, api_key):
        self.api_key = api_key
        self.mirror_url = "http://www.thetvdb.com"
        self.base_url =  self.mirror_url + "/api"
        self.base_key_url = "%s/%s" % (self.base_url, self.api_key)
        
    class Show(object):
        """A python object representing a thetvdb.com show record."""
        def __init__(self, node, mirror_url):
            # Main show details
            self.id = node.findtext("id")
            self.name = node.findtext("SeriesName")
            self.overview = node.findtext("Overview")
            self.genre = [g for g in node.findtext("Genre").split("|") if g]
            self.actors = [a for a in node.findtext("Actors").split("|") if a]
            self.network = node.findtext("Network")
            self.content_rating = node.findtext("ContentRating")
            self.rating = node.findtext("Rating")
            self.runtime = node.findtext("Runtime")
            self.status = node.findtext("Status")
            self.language = node.findtext("Language")
        
            # Air details
            self.first_aired = TheTVDB.convert_date(node.findtext("FirstAired"))
            self.airs_day = node.findtext("Airs_DayOfWeek")
            self.airs_time = TheTVDB.convert_time(node.findtext("Airs_Time"))
        
            # Main show artwork
            self.banner_url = "%s/banners/%s" % (mirror_url, node.findtext("banner"))
            self.poster_url = "%s/banners/%s" % (mirror_url, node.findtext("poster"))
            self.fanart_url = "%s/banners/%s" % (mirror_url, node.findtext("fanart"))

            # External references
            self.imdb_id = node.findtext("IMDB_ID")
            self.tvcom_id = node.findtext("SeriesID")
            self.zap2it_id = node.findtext("zap2it_id")

            # When this show was last updated
            self.last_updated = datetime.datetime.fromtimestamp(int(node.findtext("lastupdated")))
        
        def __str__(self):
            import pprint
            return pprint.saferepr(self)

    class Episode(object):
        """A python object representing a thetvdb.com episode record."""
        def __init__(self, node, mirror_url):
            self.id = node.findtext("id")
            self.show_id = node.findtext("seriesid")
            self.name = node.findtext("EpisodeName")
            self.overview = node.findtext("Overview")
            self.season_number = node.findtext("SeasonNumber")
            self.episode_number = node.findtext("EpisodeNumber")
            self.director = node.findtext("Director")
            self.guest_stars = node.findtext("GuestStars")
            self.language = node.findtext("Language")
            self.production_code = node.findtext("ProductionCode")
            self.rating = node.findtext("Rating")
            self.writer = node.findtext("Writer")

            # Air date
            self.first_aired = TheTVDB.convert_date(node.findtext("FirstAired"))
            
            # DVD Information
            self.dvd_chapter = node.findtext("DVD_chapter")
            self.dvd_disc_id = node.findtext("DVD_discid")
            self.dvd_episode_number = node.findtext("DVD_episodenumber")
            self.dvd_season = node.findtext("DVD_season")

            # Artwork/screenshot
            self.image = node.findtext("filename")

            # Episode ordering information (normally for specials)
            self.airs_after_season = node.findtext("airsafter_season")
            self.airs_before_season = node.findtext("airsbefore_season")
            self.airs_before_episode = node.findtext("airsbefore_episode")

            # Unknown
            self.combined_episode_number = node.findtext("combined_episode_number")
            self.combined_season = node.findtext("combined_season")
            self.absolute_number = node.findtext("absolute_number")
            self.season_id = node.findtext("seasonid")
            self.ep_img_flag = node.findtext("EpImgFlag")

            # External references
            self.imdb_id = node.findtext("IMDB_ID")

            # When this episode was last updated
            self.last_updated = datetime.datetime.fromtimestamp(int(node.findtext("lastupdated")))

        def __str__(self):
            return repr(self)

    @staticmethod
    def convert_time(time_string):
        """Convert a thetvdb time string into a datetime.time object."""
        time_res = [re.compile(r"\D*(?P<hour>\d{1,2})(?::(?P<minute>\d{2}))?.*(?P<ampm>a|p)m.*", re.IGNORECASE), # 12 hour
                    re.compile(r"\D*(?P<hour>\d{1,2}):?(?P<minute>\d{2}).*")]                                     # 24 hour

        for r in time_res:
            m = r.match(time_string)
            if m:
                gd = m.groupdict()

                if "hour" in gd and "minute" in gd and gd["minute"] and "ampm" in gd:
                    hour = int(gd["hour"])
                    if gd["ampm"].lower() == "p":
                        hour += 12

                    return datetime.time(hour, int(gd["minute"]))
                elif "hour" in gd and "ampm" in gd:
                    hour = int(gd["hour"])
                    if gd["ampm"].lower() == "p":
                        hour += 12

                    return datetime.time(hour, 0)
                elif "hour" in gd and "minute" in gd:
                    return datetime.time(int(gd["hour"]), int(gd["minute"]))

        return None

    @staticmethod
    def convert_date(date_string):
        """Convert a thetvdb date string into a datetime.date object."""
        first_aired = None
        try:
            first_aired = datetime.date(*map(int, date_string.split("-")))
        except ValueError:
            pass

        return first_aired

    def get_matching_shows(self, show_name):
        """Get a list of shows matching show_name."""
        get_args = urllib.urlencode({"seriesname": show_name}, doseq=True)
        url = "%s/GetSeries.php?%s" % (self.base_url, get_args)
        data = urllib.urlopen(url)
        show_list = []

        if data:
            try:
                tree = ET.parse(data)
                show_list = [(show.findtext("seriesid"), show.findtext("SeriesName")) for show in tree.getiterator("Series")]
            except SyntaxError:
                pass

        return show_list

    def get_show(self, show_id):
        """Get the show object matching this show_id."""
        url = "%s/series/%s/" % (self.base_key_url, show_id)
        data = urllib.urlopen(url)
        
        show = None
        try:
            tree = ET.parse(data)
            show_node = tree.find("Series")
        
            show = TheTVDB.Show(show_node, self.mirror_url)
        except SyntaxError:
            pass
        
        return show

    def get_episode(self, episode_id):
        """Get the episode object matching this episode_id."""
        url = "%s/episodes/%s/" % (self.base_key_url, episode_id)
        data = urllib.urlopen(url)
        
        episode = None
        try:
            tree = ET.parse(data)
            episode_node = tree.find("Episode")

            episode = TheTVDB.Episode(episode_node, self.mirror_url)
        except SyntaxError:
            pass
        
        return episode
    
    def get_show_and_episodes(self, show_id):
        """Get the show object and all matching episode objects for this show_id."""
        url = "%s/series/%s/all/" % (self.base_key_url, show_id)
        data = urllib.urlopen(url)
        
        show_and_episodes = None
        try:
            tree = ET.parse(data)
            show_node = tree.find("Series")
        
            show = TheTVDB.Show(show_node, self.mirror_url)
            episodes = []
        
            episode_nodes = tree.getiterator("Episode")
            for episode_node in episode_nodes:
                episodes.append(TheTVDB.Episode(episode_node, self.mirror_url))
        
            show_and_episodes = (show, episodes)
        except SyntaxError:
            pass
        
        return show_and_episodes

    def get_updated_shows(self, period = "day"):
        """Get a list of show ids which have been updated within this period."""
        url = "%s/updates/updates_%s.xml" % (self.base_key_url, period)
        data = urllib.urlopen(url)
        tree = ET.parse(data)

        series_nodes = tree.getiterator("Series")

        return [x.findtext("id") for x in series_nodes]

    def get_updated_episodes(self, period = "day"):
        """Get a list of episode ids which have been updated within this period."""
        url = "%s/updates/updates_%s.xml" % (self.base_key_url, period)
        data = urllib.urlopen(url)
        tree = ET.parse(data)

        episode_nodes = tree.getiterator("Episode")

        return [(x.findtext("Series"), x.findtext("id")) for x in episode_nodes]

    def get_show_image_choices(self, show_id):
        """Get a list of image urls and types relating to this show."""
        url = "%s/series/%s/banners.xml" % (self.base_key_url, show_id)
        data = urllib.urlopen(url)
        tree = ET.parse(data)

        images = []

        banner_data = tree.find("Banners")
        banner_nodes = tree.getiterator("Banner")
        for banner in banner_nodes:
            banner_path = banner.findtext("BannerPath")
            banner_type = banner.findtext("BannerType")
            banner_url = "%s/banners/%s" % (self.mirror_url, banner_path)

            images.append((banner_url, banner_type))

        return images


#!/usr/bin/env python2.5
#encoding:utf-8
#author:dbr/Ben
#contributors: by ccjensen/Chris
#project:themoviedb
#repository:http://github.com/dbr/tvdb_api
#license: LGPLv2 http://www.gnu.org/licenses/lgpl.html

"""An interface to the themoviedb.org API
"""

__author__ = "dbr/Ben"
__version__ = "0.2b"

config = {}
config['apikey'] = "a8b9f96dde091408a03cb4c78477bd14"

config['urls'] = {}
config['urls']['movie.search'] = "http://api.themoviedb.org/2.1/Movie.search/en/xml/%(apikey)s/%%s" % (config)
config['urls']['movie.getInfo'] = "http://api.themoviedb.org/2.1/Movie.getInfo/en/xml/%(apikey)s/%%s" % (config)
config['urls']['media.getInfo'] = "http://api.themoviedb.org/2.1/Media.getInfo/en/xml/%(apikey)s/%%s/%%s" % (config)


import os
import struct
import urllib
import urllib2

import xml.etree.cElementTree as ElementTree


class TmdBaseError(Exception):
    pass


class TmdNoResults(TmdBaseError):
    pass

class TmdHttpError(TmdBaseError):
    pass


class TmdXmlError(TmdBaseError):
    pass


def opensubtitleHashFile(name):
    """Hashes a file using OpenSubtitle's method.

    > In natural language it calculates: size + 64bit chksum of the first and
    > last 64k (even if they overlap because the file is smaller than 128k).

    A slightly more Pythonic version of the Python solution on..
    http://trac.opensubtitles.org/projects/opensubtitles/wiki/HashSourceCodes
    """
    longlongformat = 'q'
    bytesize = struct.calcsize(longlongformat)

    f = open(name, "rb")

    filesize = os.path.getsize(name)
    fhash = filesize

    if filesize < 65536 * 2:
       raise ValueError("File size must be larger than %s bytes (is %s)" % (65536*2, filesize))

    for x in range(65536/bytesize):
        buf = f.read(bytesize)
        (l_value,)= struct.unpack(longlongformat, buf)
        fhash += l_value
        fhash = fhash & 0xFFFFFFFFFFFFFFFF # to remain as 64bit number


    f.seek(max(0,filesize-65536),0)
    for x in range(65536/bytesize):
        buf = f.read(bytesize)
        (l_value,)= struct.unpack(longlongformat, buf)
        fhash += l_value
        fhash = fhash & 0xFFFFFFFFFFFFFFFF

    f.close()
    return  "%016x" % fhash


class XmlHandler:
    """Deals with retrieval of XML files from API
    """

    def __init__(self, url):
        self.url = url

    def _grabUrl(self, url):
        try:
            urlhandle = urllib2.urlopen(url)
        except IOError, errormsg:
            raise TmdHttpError(errormsg)
        if urlhandle.code >= 400:
            raise TmdHttpError("HTTP status code was %d" % urlhandle.code)
        return urlhandle.read()

    def getEt(self):
        xml = self._grabUrl(self.url)
        try:
            et = ElementTree.fromstring(xml)
        except SyntaxError, errormsg:
            raise TmdXmlError(errormsg)
        return et


class SearchResults(list):
    """Stores a list of Movie's that matched the search
    """

    def __repr__(self):
        return "<Search results: %s>" % (list.__repr__(self))


class MovieResult(dict):
    """A dict containing the information about a specific search result
    """

    def __repr__(self):
        return "<MovieResult: %s (%s)>" % (self.get("name"), self.get("released"))

    def info(self):
        """Performs a MovieDb.getMovieInfo search on the current id, returns
        a Movie object
        """
        cur_id = self['id']
        info = MovieDb().getMovieInfo(cur_id)
        return info

class Movie(dict):
    """A dict containing the information about the film
    """

    def __repr__(self):
        return "<MovieResult: %s (%s)>" % (self.get("name"), self.get("released"))


class Categories(dict):
    """Stores category information
    """

    def set(self, category_et):
        """Takes an elementtree Element ('category') and stores the url,
        using the type and name as the dict key.

        For example:
       <category type="genre" url="http://themoviedb.org/encyclopedia/category/80" name="Crime"/>

        ..becomes:
        categories['genre']['Crime'] = 'http://themoviedb.org/encyclopedia/category/80'
        """
        _type = category_et.get("type")
        name = category_et.get("name")
        url = category_et.get("url")
        self.setdefault(_type, {})[name] = url
        self[_type][name] = url


class Studios(dict):
    """Stores category information
    """

    def set(self, studio_et):
        """Takes an elementtree Element ('studio') and stores the url,
        using the name as the dict key.

        For example:
       <studio url="http://www.themoviedb.org/encyclopedia/company/20" name="Miramax Films"/>

        ..becomes:
        studios['name'] = 'http://www.themoviedb.org/encyclopedia/company/20'
        """
        name = studio_et.get("name")
        url = studio_et.get("url")
        self[name] = url


class Countries(dict):
    """Stores country information
    """

    def set(self, country_et):
        """Takes an elementtree Element ('country') and stores the url,
        using the name and code as the dict key.

        For example:
       <country url="http://www.themoviedb.org/encyclopedia/country/223" name="United States of America" code="US"/>

        ..becomes:
        countries['code']['name'] = 'http://www.themoviedb.org/encyclopedia/country/223'
        """
        code = country_et.get("code")
        name = country_et.get("name")
        url = country_et.get("url")
        self.setdefault(code, {})[name] = url


class Image(dict):
    """Stores image information for a single poster/backdrop (includes
    multiple sizes)
    """

    def __init__(self, _id, _type, size, url):
        self['id'] = _id
        self['type'] = _type

    def largest(self):
        for csize in ["original", "mid", "cover", "thumb"]:
            if csize in self:
                return csize

    def __repr__(self):
        return "<Image (%s for ID %s)>" % (self['type'], self['id'])


class ImagesList(list):
    """Stores a list of Images, and functions to filter "only posters" etc
    """

    def set(self, image_et):
        """Takes an elementtree Element ('image') and stores the url,
        along with the type, id and size.

        Is a list containing each image as a dictionary (which includes the
        various sizes)

        For example:
        <image type="poster" size="original" url="http://images.themoviedb.org/posters/4181/67926_sin-city-02-color_122_207lo.jpg" id="4181"/>

        ..becomes:
        images[0] = {'id':4181', 'type': 'poster', 'original': 'http://images.themov...'}
        """
        _type = image_et.get("type")
        _id = image_et.get("id")
        size = image_et.get("size")
        url = image_et.get("url")

        cur = self.find_by('id', _id)
        if len(cur) == 0:
            nimg = Image(_id = _id, _type = _type, size = size, url = url)
            self.append(nimg)
        elif len(cur) == 1:
            cur[0][size] = url
        else:
            raise ValueError("Found more than one poster with id %s, this should never happen" % (_id))

    def find_by(self, key, value):
        ret = []
        for cur in self:
            if cur[key] == value:
                ret.append(cur)
        return ret

    @property
    def posters(self):
        return self.find_by('type', 'poster')

    @property
    def backdrops(self):
        return self.find_by('type', 'backdrop')


class CrewRoleList(dict):
    """Stores a list of roles, such as director, actor etc

    >>> import tmdb
    >>> tmdb.getMovieInfo(550)['cast'].keys()[:5]
    ['casting', 'producer', 'author', 'sound editor', 'actor']
    """
    pass


class CrewList(list):
    """Stores list of crew in specific role

    >>> import tmdb
    >>> tmdb.getMovieInfo(550)['cast']['author']
    [<author (id 7468): Chuck Palahniuk>, <author (id 7469): Jim Uhls>]
    """
    pass


class Person(dict):
    """Stores information about a specific member of cast
    """

    def __init__(self, job, _id, name, character, url):
        self['job'] = job
        self['id'] = _id
        self['name'] = name
        self['character'] = character
        self['url'] = url

    def __repr__(self):
        if self['character'] is None or self['character'] == "":
            return "<%(job)s (id %(id)s): %(name)s>" % self
        else:
            return "<%(job)s (id %(id)s): %(name)s (as %(character)s)>" % self


class MovieDb:
    """Main interface to www.themoviedb.com

    The search() method searches for the film by title.
    The getMovieInfo() method retrieves information about a specific movie using themoviedb id.
    """

    def _parseSearchResults(self, movie_element):
        cur_movie = MovieResult()
        cur_images = ImagesList()
        for item in movie_element.getchildren():
                if item.tag.lower() == "images":
                    for subitem in item.getchildren():
                        cur_images.set(subitem)
                else:
                    cur_movie[item.tag] = item.text
        cur_movie['images'] = cur_images
        return cur_movie

    def _parseMovie(self, movie_element):
        cur_movie = Movie()
        cur_categories = Categories()
        cur_studios = Studios()
        cur_countries = Countries()
        cur_images = ImagesList()
        cur_cast = CrewRoleList()
        for item in movie_element.getchildren():
            if item.tag.lower() == "categories":
                for subitem in item.getchildren():
                    cur_categories.set(subitem)
            elif item.tag.lower() == "studios":
                for subitem in item.getchildren():
                    cur_studios.set(subitem)
            elif item.tag.lower() == "countries":
                for subitem in item.getchildren():
                    cur_countries.set(subitem)
            elif item.tag.lower() == "images":
                for subitem in item.getchildren():
                    cur_images.set(subitem)
            elif item.tag.lower() == "cast":
                for subitem in item.getchildren():
                    job = subitem.get("job").lower()
                    p = Person(
                        job = job,
                        _id = subitem.get("id"),
                        name = subitem.get("name"),
                        character = subitem.get("character"),
                        url = subitem.get("url"),
                    )
                    cur_cast.setdefault(job, CrewList()).append(p)
            else:
                cur_movie[item.tag] = item.text

        cur_movie['categories'] = cur_categories
        cur_movie['studios'] = cur_studios
        cur_movie['countries'] = cur_countries
        cur_movie['images'] = cur_images
        cur_movie['cast'] = cur_cast
        return cur_movie

    def search(self, title):
        """Searches for a film by its title.
        Returns SearchResults (a list) containing all matches (Movie instances)
        """
        title = urllib.quote(title.encode("utf-8"))
        url = config['urls']['movie.search'] % (title)
        etree = XmlHandler(url).getEt()
        search_results = SearchResults()
        for cur_result in etree.find("movies").findall("movie"):
            cur_movie = self._parseSearchResults(cur_result)
            search_results.append(cur_movie)
        return search_results

    def getMovieInfo(self, id):
        """Returns movie info by it's TheMovieDb ID.
        Returns a Movie instance
        """
        url = config['urls']['movie.getInfo'] % (id)
        etree = XmlHandler(url).getEt()
        moviesTree = etree.find("movies").findall("movie")

        if len(moviesTree) == 0:
            raise TmdNoResults("No results for id %s" % id)

        return self._parseMovie(moviesTree[0])

    def mediaGetInfo(self, hash, size):
        """Used to retrieve specific information about a movie but instead of
        passing a TMDb ID, you pass a file hash and filesize in bytes
        """
        url = config['urls']['media.getInfo'] % (hash, size)
        etree = XmlHandler(url).getEt()
        moviesTree = etree.find("movies").findall("movie")
        if len(moviesTree) == 0:
            raise TmdNoResults("No results for hash %s" % hash)

        return [self._parseMovie(x) for x in moviesTree]


def search(name):
    """Convenience wrapper for MovieDb.search - so you can do..

    >>> import tmdb
    >>> tmdb.search("Fight Club")
    <Search results: [<MovieResult: Fight Club (1999-09-16)>]>
    """
    mdb = MovieDb()
    return mdb.search(name)


def getMovieInfo(id):
    """Convenience wrapper for MovieDb.search - so you can do..

    >>> import tmdb
    >>> tmdb.getMovieInfo(187)
    <MovieResult: Sin City (2005-04-01)>
    """
    mdb = MovieDb()
    return mdb.getMovieInfo(id)


def mediaGetInfo(hash, size):
    """Convenience wrapper for MovieDb.mediaGetInfo - so you can do..

    >>> import tmdb
    >>> tmdb.mediaGetInfo('907172e7fe51ba57', size = 742086656)[0]
    <MovieResult: Sin City (2005-04-01)>
    """
    mdb = MovieDb()
    return mdb.mediaGetInfo(hash, size)


def searchByHashingFile(filename):
    """Searches for the specified file using the OpenSubtitle hashing method
    """
    return mediaGetInfo(opensubtitleHashFile(filename), os.path.size(filename))


def main():
    results = search("Fight Club")
    searchResult = results[0]
    movie = getMovieInfo(searchResult['id'])
    print movie['name']

    print "Producers:"
    for prodr in movie['cast']['producer']:
        print " " * 4, prodr['name']
    print movie['images']
    for genreName in movie['categories']['genre']:
        print "%s (%s)" % (genreName, movie['categories']['genre'][genreName])


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


__version__ = '0.2'

import logging

class NullHandler(logging.Handler):
    def emit(self, record):
        pass

h = NullHandler()
logging.getLogger("smewt").addHandler(h)


from base import SmewtDict, ValidatingSmewtDict, SmewtException, SmewtUrl, SolvingChain, cachedmethod, EventServer, cache, utils, textutils
from base.mediaobject import Media, Metadata
log = logging.getLogger('smewt')


# used to be able to store settings for different versions of Smewt installed on the same computer, ie: a stable
# and a development version
ORG_NAME = 'Falafelton'
APP_NAME = 'Smewt'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import SIGNAL, QObject
from smewt.base import SmewtException
from smewt.base.mediaobject import Media

class Guesser(QObject):
    """Abstract class from which all guessers must inherit.  Guessers are objects
    that implement a slot called start(self, query) that returns immediately, and
    begins the process of guessing metadata of the first element of the given
    Collection.media list.

    When all guesses are made it emits a signal called finished(guesses) which
    returns the original Collection augmented with the guesses it could have made.

    The following needs to be defined in derived classes:

    1- 'supportedTypes' which lists the media types for which this guesser can
        provide metadata.

    """

    def __init__(self):
        super(Guesser, self).__init__()

    def checkValid(self, query):
        '''Checks that we have only one object in Collection.media list and that
        its type is supported by our guesser'''
        media = query.find_all(node_type = Media)
        if len(media) != 1:
            raise SmewtException('Guesser: your query should contain exactly 1 element in the Collection.media list')

        if media[0].type() not in self.supportedTypes:
            raise SmewtException('Guesser: this guesser only supports files of type: %s but you provided a file of type: %s' % (str(self.supportedTypes), media[0].type()))


    def start(self, query):
        self.emit(SIGNAL('finished'), query)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewtexception import SmewtException
import logging

log = logging.getLogger('smewt.base.solvingchain')

class SolvingChain(object):
    def __init__(self, *args):
        super(SolvingChain, self).__init__()

        self.chain = args
        if not args:
            raise SmewtException('Tried to build an empty solving chain')

    def solve(self, query):
        result = query
        for action in self.chain:
            log.debug("SolvingChain: performing action %s" % action.__class__.__name__)
            result = action.perform(result)
        return result




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import Metadata

class Series(Metadata):

    #typename = 'Series'

    schema = { 'title': unicode,
               'numberSeasons': int,
               #'episodeList': list
               }

    valid = [ 'title' ]
    unique = [ 'title' ]

    #converters = { 'episodeList': lambda x:x } #parseEpisodeList }


class Episode(Metadata):

    #typename = 'Episode'

    schema = { 'series': Series,
               'season': int,
               'episodeNumber': int,
               'title': unicode
               }

    valid = [ 'series', 'season', 'episodeNumber' ]
    reverse_lookup = { 'series': 'episodes' }
    #order = [ 'series', 'season', 'episodeNumber',  'title' ]

    unique = [ 'series', 'season', 'episodeNumber' ]

    converters = {}




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

'''
NOTES:

 - the API should be aware that there might be different releases for the same file (eg: multiple
   release groups). It should return as much info as possible so that we might try to match this
   info with the one we have from the movie itself

 - the API should be aware that some subtitles might span more than 1 file (eg: multiple CDs movies)

 - if the source of the subtitles allows downloading all of the subs for a season at once, this should
   be cached in the implementation, but should not appear in the API.

TODO: what about the encoding? utf-8 sounds better, but IIRC mplayer prefers iso-8859-1...
'''

from smewt.base import SmewtException

class SubtitleNotFoundError(SmewtException):
    pass



class SubtitleProvider:
    """This class represents the interface that needs to be implemented by a plugin which downloads
    subtitles for movies or series."""

    def canHandle(self, metadata):
        """This method returns whether this subtitle provider is able to handle the given object.

        Only objects which fulfill this condition should be handed to the getAvailableSubtitles
        method."""

        raise NotImplementedError


    def getAvailableSubtitles(self, metadata):
        """This method should return a graph of all available Subtitle metadata objects for
        the given Metadata (Movie, Episode, ...) object.

        If no subtitle could be found, this method should return an empty graph.

        It is assumed that the given Metadata object complies with the canHandle method.

        NB: This graph can contain multiple versions of the subtitle (different languages,
        different rips, ...) for the same object.
        The subtitles may also span multiple files (if the object (movie) is split in the same way)."""

        raise NotImplementedError


    def getSubtitle(self, subtitle):
        """This method should return the contents of the given subtitle as a string.
        The Subtitle object should be picked among the ones returned by the getAvailableSubtitles
        method.

        If the subtitle could not be found, a SubtitleNotFoundError exception should be raised."""

        raise NotImplementedError



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from Cheetah.Template import Template
from pygoo import MemoryObjectGraph, ontology
from smewt.base import SmewtException, Media
from movieobject import Movie
from smewt.base.utils import smewtDirectory
ontology.import_class('Movie')

def render(url, collection):
    '''This function always receive an URL and a full graph of all the collection as metadata input.
    This is the place to put some logic before the html rendering is done, such as filtering out
    items we don't want to display, or shape the data so that it's more suited for html rendering, etc...'''

    # TODO: this is the place where we know which data we want to cache from the database to a MemoryObjectGraph

    if url.viewType == 'single':
        t = Template(file = smewtDirectory('smewt', 'media', 'movie', 'view_movie.tmpl'),
                     searchList = { 'movie': collection.find_one(Movie, title = url.args['title']) })

    elif url.viewType == 'all':
        t = Template(file = smewtDirectory('smewt', 'media', 'movie', 'view_all_movies.tmpl'),
                     searchList = { 'movies': collection.find_all(Movie) })

    elif url.viewType == 'spreadsheet':
        t = Template(file = smewtDirectory('smewt', 'media', 'movie', 'view_movies_spreadsheet.tmpl'),
                     searchList = { 'movies': collection.find_all(Movie),
                                    'title': 'ALL' })

    elif url.viewType == 'unwatched':
        t = Template(file = smewtDirectory('smewt', 'media', 'movie', 'view_movies_spreadsheet.tmpl'),
                     searchList = { 'movies': [ m for m in collection.find_all(node_type = Movie) if not m.get('watched') ],
                                    'title': 'UNWATCHED' })

    elif url.viewType == 'recent':
        t = Template(file = smewtDirectory('smewt', 'media', 'movie', 'view_recent_movies.tmpl'),
                     searchList = { 'movies': [ m for m in collection.find_all(node_type = Movie) if m.get('lastViewed') is not None ],
                                    'title': 'RECENT' })

    else:
        raise SmewtException('Invalid view type: %s' % url.viewType)

    result = t.respond()
    #open('/tmp/view.html', 'w').write(result.encode('utf-8'))
    return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import GraphAction, Metadata
from smewt.base.mediaobject import foundMetadata
import logging

log = logging.getLogger('smewt.solvers.solver')

class Solver(GraphAction):
    def __init__(self):
        super(Solver, self).__init__()

    def canHandle(self, query):
        try:
            query.find_one(Metadata)
        except:
            raise SmewtException('%s: not solving anything...' % self.__class__.__name__)

        log.debug('%s: trying to solve %s' % (self.__class__.__name__, query))

    def found(self, query, result):
        log.debug('%s: found for %s: %s' % (self.__class__.__name__, query, result))

        solved = foundMetadata(query, result)
        #solved.displayGraph()
        return solved


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import feedparser
import urllib2, re
import subprocess
from PyQt4.QtCore import SIGNAL, Qt, QSettings, QVariant, QAbstractListModel
from smewt import SmewtException, EventServer

class AmuleFeedWatcher(QAbstractListModel):
    def __init__(self):
        super(AmuleFeedWatcher, self).__init__()
        self.loadFeeds()

    def rowCount(self, parent):
        return len(self.feedList)

    def data(self, index, role = Qt.DisplayRole):
        if role != Qt.DisplayRole:
            return QVariant()

        f = self.feedList[index.row()]
        return QVariant(f['title'] + '\n   -- last episode: ' + f['lastTitle'] + '\n')

    def feedListToQVariant(self, feedList):
        return QVariant([ QVariant([ QVariant(f['url']),
                                     QVariant(f['title']),
                                     QVariant([ QVariant(float(n)) for n in f['lastUpdate'] ]),
                                     QVariant(f['lastTitle'])
                                     ]) for f in feedList ])

    def variantToFeedList(self, v):
        result = []
        for f in v.toList():
            feed = {}
            feed['url'] = str(f.toList()[0].toString())
            feed['title'] = str(f.toList()[1].toString())
            feed['lastUpdate'] = [ n.toInt()[0] for n in f.toList()[2].toList() ]
            feed['lastTitle'] = unicode(f.toList()[3].toString())
            result.append(feed)
        return result

    def loadFeeds(self):
        self.feedList = self.variantToFeedList(QSettings().value('feeds'))

    def saveFeeds(self):
        QSettings().setValue('feeds', self.feedListToQVariant(self.feedList))

    def addFeed(self, url, lastUpdate = None):
        """Example:
        url = 'http://tvu.org.ru/rss.php?se_id=19934' # South Park season 12
        lastUpdate = [2008, 10, 30, 2, 47, 39, 3, 304, 0]
        addFeed(url, lastUpdate)

        if lastUpdate is not specified, it will assume all episodes have already been downloaded"""

        url = str(url)
        if url not in [ f['url'] for f in self.feedList ]:
            try:
                pfeed = feedparser.parse(url)
                feed = { 'url': url,
                         'title': pfeed.channel.title }
            except AttributeError:
                raise SmewtException('Invalid feed!')

            if not lastUpdate:
                lastUpdate = list(pfeed.entries[0].updated_parsed)

            self.feedList.append(feed)
            self.setLastUpdate(feed, lastUpdate)

            self.saveFeeds()

            # FIXME: we should emit dataChanged here rather than call reset() (same goes for removeFeed)
            self.reset()

    def removeFeed(self, url):
        for f in self.feedList:
            if f['url'] == url:
                self.feedList.remove(f)
                self.saveFeeds()
                self.reset()

    def removeFeedIndex(self, idx):
        self.removeFeed(self.feedList[idx]['url'])

    def getFullFeedIndex(self, idx):
        feed = self.feedList[idx]
        feeds = feedparser.parse(feed['url'])
        entries = [ { 'title': entry.title,
                      'updated': list(entry.updated_parsed) } for entry in feeds.entries ]

        return { 'title': feed['title'],
                 'lastUpdate': feed['lastUpdate'],
                 'entries': entries }

    def setLastUpdateIndex(self, index, lastUpdate):
        feed = self.feedList[index]
        self.setLastUpdate(feed, lastUpdate)

    def setLastUpdate(self, feed, lastUpdate):
        feed['lastUpdate'] = lastUpdate
        # also save last ep title:
        index = self.feedList.index(feed)
        for f in self.getFullFeedIndex(index)['entries']:
            if lastUpdate == f['updated']:
                feed['lastTitle'] = f['title']
                break
        self.saveFeeds()
        self.reset()


    def amuleDownload(self, ed2kLink):
        from amulecommand import AmuleCommand
        return AmuleCommand().download(ed2kLink)

    def downloadNewEpisodes(self, feed):
        EventServer.publish('Checking new episodes for: %s' % feed['title'])
        f = feedparser.parse(feed['url'])
        lastUpdate = feed['lastUpdate']

        for ep in f.entries:
            if list(ep.updated_parsed) > feed['lastUpdate']:
                EventServer.publish('Found new episode: %s' % ep.title)
                episodeHtml = urllib2.urlopen(ep.id).read()
                ed2kLink = re.compile('href="(?P<url>ed2k://\|file.*?)">').search(episodeHtml).groups()[0]
                EventServer.publish('Sending file to aMule...')
                ok, msg = self.amuleDownload(ed2kLink)
                if ok:
                    EventServer.publish('Successfully sent to aMule!')
                    if list(ep.updated_parsed) > lastUpdate:
                        lastUpdate = list(ep.updated_parsed)
                else:
                    EventServer.publish('Error while sending to aMule. %s. Will try again next time...' % msg)

        if lastUpdate == feed['lastUpdate']:
            EventServer.publish('No new episodes...')
        else:
            self.setLastUpdate(feed, lastUpdate)

    def checkAllFeeds(self):
        for feed in self.feedList:
            self.downloadNewEpisodes(feed)

        # write back feed list with latest updates
        self.saveFeeds()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from episodefilename import EpisodeFilename
from episodetvdb import EpisodeTVDB
from moviefilename import MovieFilename
from movietmdb import MovieTMDB





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2011 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging
import time, os

from PyQt4.QtCore import QObject, SIGNAL, QVariant, QSettings
from pygoo import MemoryObjectGraph, ontology

from smewt.base import utils, Media, Metadata
from smewt.base import ImportTask, SubtitleTask
from smewt.taggers import EpisodeTagger, MovieTagger
from smewt.base.textutils import toUtf8

log = logging.getLogger('smewt.base.collection')

class Collection(object):
    """A Collection keeps track of the files in the folders for us. when any changes
    happen there, it creates new import tasks for new files or removes references to
    deleted files."""

    def __init__(self, name, validFiles, mediaTagger, dataGraph, taskManager, folders = {}):
        # identifies the collection
        self.name = name
        # validFiles is a list of conditions a file should meet to be considered to belong to this collection
        self.validFiles = validFiles
        self.mediaTagger = mediaTagger

        self.graph = dataGraph
        self.taskManager = taskManager

        # folders is a dict of folder names to bool indicating whether they should be traversed recursively
        self.folders = folders

        self.loadSettings()


    def loadSettings(self):
        settings = QSettings()
        collection = settings.value('collection_%s' % self.name)
        if collection:
            self.folders = dict((toUtf8(folder), recursive.toBool()) for folder, recursive in collection.toMap().items())
            log.info('loaded %s folders: %s' % (self.name, self.folders))

    def saveSettings(self):
        QSettings().setValue('collection_%s' % self.name, QVariant(self.folders))


    def checkIntegrity():
        # delete files in set(files) - set(dirwalk())
        pass


    def setFolders(self, folders):
        self.folders = folders

        self.saveSettings()
        self.update()


    def collectionFiles(self):
        for folder, recursive in self.folders.items():
            for f in utils.dirwalk(folder, self.validFiles, recursive):
                yield f.decode('utf-8')

    def modifiedFiles(self):
        lastModified = dict((f.filename, f.get('lastModified', None)) for f in self.graph.find_all(Media))
        for f in self.collectionFiles():
            # yield a file if we haven't heard of it yet or if it has been modified recently
            if f not in lastModified or os.path.getmtime(f) > lastModified[f]:
                yield f

    def importFiles(self, files):
        for f in files:
            # new import task
            log.info('Import in %s collection: %s' % (self.name, toUtf8(f)))
            if self.taskManager:
                importTask = ImportTask(self.graph, self.mediaTagger, f)
                self.taskManager.add(importTask)

        # save newly imported files
        self.saveSettings()


    def update(self):
        log.info('Updating %s collection' % self.name)
        self.importFiles(self.modifiedFiles())

    def rescan(self):
        log.info('Rescanning %s collection' % self.name)
        self.importFiles(self.collectionFiles())





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt import SmewtException, SmewtUrl, Media, Metadata
from smewt.gui.collectionfolderspage import CollectionFoldersPage
from smewt.media import Series, Episode, Movie
from smewt.base import ImportTask, SubtitleTask, ActionFactory
from smewt.base.taskmanager import Task, TaskManager
from PyQt4.QtCore import SIGNAL, SLOT, QVariant, QProcess, QSettings, pyqtSignature
from PyQt4.QtGui import QWidget, QPushButton, QHBoxLayout, QVBoxLayout, QFileDialog, QSizePolicy
from PyQt4.QtWebKit import QWebView, QWebPage
from smewt.media import series, movie, speeddial
import logging
import time
from os.path import join, dirname, splitext
from smewt.taggers import EpisodeTagger, MovieTagger
from smewt.base import SmewtDaemon

log = logging.getLogger('smewt.gui.mainwidget')

minZoomFactor = 0.5
maxZoomFactor = 3.0
stepZoomFactor = 0.1

class MainWidget(QWidget):
    def __init__(self):
        super(MainWidget, self).__init__()

        self.collectionView = QWebView()
        self.collectionView.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        #self.collectionView.page().setLinkDelegationPolicy(QWebPage.DelegateExternalLinks)
        self.setZoomFactor(QSettings().value('zoom_factor', QVariant(1.0)).toDouble()[0])
        self.connect(self.collectionView,  SIGNAL('linkClicked(const QUrl&)'),
                     self.linkClicked)

        layout = QVBoxLayout()
        layout.addWidget(self.collectionView)

        self.smewtd = SmewtDaemon()
        self.smewtd.taskManager.progressChanged.connect(self.progressChanged)

        self.setLayout(layout)

        self.history = []
        self.index = 0
        baseUrl = QSettings().value('base_url').toString()
        if baseUrl == '':
            baseUrl = 'smewt://media/speeddial/'
        self.setSmewtUrl(baseUrl)

        self.externalProcess = QProcess()


    def progressChanged(self, finished, total):
        if total == 0:
            self.refreshCollectionView()

    def quit(self):
        self.smewtd.quit()

    def setZoomFactor(self, factor):
        self.collectionView.page().mainFrame().setTextSizeMultiplier( factor )

    def zoomIn(self):
        zoomFactor = min(QSettings().value('zoom_factor', QVariant(1.0)).toDouble()[0] + stepZoomFactor, maxZoomFactor )
        QSettings().setValue('zoom_factor', QVariant( zoomFactor ) )

        self.setZoomFactor( zoomFactor )

    def zoomOut(self):
        zoomFactor = max(QSettings().value('zoom_factor', QVariant(1.0)).toDouble()[0] - stepZoomFactor, minZoomFactor)
        QSettings().setValue('zoom_factor', QVariant( zoomFactor ) )

        self.setZoomFactor( zoomFactor )

    def back(self):
        self.setSmewtUrl(None, self.index - 1)

    def forward(self):
        self.setSmewtUrl(None, self.index + 1)

    def speedDial(self):
        self.setSmewtUrl(SmewtUrl('media', 'speeddial/'))

    def setSmewtUrl(self, url, index = None):
        if index is not None:
            self.index = min(max(index, 0), len(self.history)-1)
            self.smewtUrl = self.history[self.index]

        else:
            if not isinstance(url, SmewtUrl):
                url = SmewtUrl(url = url)

            self.smewtUrl = url

            self.history[self.index+1:] = []
            self.history.append(url)
            self.index = len(self.history) - 1

        QSettings().setValue('base_url',  QVariant(unicode(self.smewtUrl)))
        try:
            self.refreshCollectionView()
        except Exception, e:
            import sys, traceback
            log.warning('Exception:\n%s' % ''.join(traceback.format_exception(*sys.exc_info())))

            # In case of error, return to the home screen
            log.warning('Returning to Speed Dial view')
            self.speedDial()

    def loadCollection(self):
        """Debug method."""
        filename = str(QFileDialog.getOpenFileName(self, 'Select file to load the collection'))
        self.collection.load(filename)


    def updateCollectionSettings(self, result):
        if result == 1:
            self.updateCollection()

    def updateCollection(self):
        self.smewtd.updateCollections()

    def rescanCollection(self):
        self.smewtd.rescanCollections()

    def selectSeriesFolders(self):
        d = CollectionFoldersPage(self,
                                  description = 'Select the folders where your series are.',
                                  collection = self.smewtd.episodeCollection)
        d.exec_()

    def selectMoviesFolders(self):
        d = CollectionFoldersPage(self,
                                  description = 'Select the folders where your movies are.',
                                  collection = self.smewtd.movieCollection)
        d.exec_()


    def mergeCollection(self, result):
        self.collection += result

    def refreshCollectionView(self):
        surl = self.smewtUrl

        if surl.mediaType == 'speeddial':
            html = speeddial.view.render(surl, self.smewtd.database)
            self.collectionView.page().mainFrame().setHtml(html)

        elif surl.mediaType == 'series':
            html = series.view.render(surl, self.smewtd.database)
            #open('/tmp/smewt.html',  'w').write(html.encode('utf-8'))
            #print html[:4000]
            self.collectionView.page().mainFrame().setHtml(html)

        elif surl.mediaType == 'movie':
            html = movie.view.render(surl,  self.smewtd.database)
            #open('/tmp/smewt.html',  'w').write(html.encode('utf-8'))
            self.collectionView.page().mainFrame().setHtml(html)
            # insert listener object for checkboxes inside the JS environment
            self.connect(self.collectionView.page().mainFrame(), SIGNAL('javaScriptWindowObjectCleared()'),
                         self.connectJavaScript)

        else:
            raise SmewtException('MainWidget: Invalid media type: %s' % surl.mediaType)

    def connectJavaScript(self):
        self.collectionView.page().mainFrame().addToJavaScriptWindowObject('mainWidget', self)

    @pyqtSignature("QString, bool")
    def updateWatched(self, title, watched):
        self.smewtd.database.find_one(Movie, title = unicode(title)).watched = watched

    @pyqtSignature("QString, QString, QString")
    def addComment(self, title, author, comment):
        g = self.smewtd.database
        movie = g.find_one(Movie, title = unicode(title))
        commentObj = g.Comment(metadata = movie,
                               author = unicode(author),
                               date = int(time.time()),
                               text = unicode(comment))

        self.refreshCollectionView()

    def linkClicked(self,  url):
        log.info('clicked on link %s', url)
        url = url.toEncoded()

        if url.startsWith('file://'):
            # FIXME: we should not use this anymore but a SmewtUrl with action = play
            log.warning('Deprecated api: links with file:// ...')
            action = 'smplayer'
            args = [ str(url)[7:] ]
            log.debug('launching %s with args = %s', (action, args))
            self.externalProcess.startDetached(action, args)

        elif url.startsWith('smewt://'):
            surl = SmewtUrl(url = url)
            if surl.mediaType:
                self.setSmewtUrl(surl)

            elif surl.actionType:
                ActionFactory().dispatch(self, surl)

            else:
                # probably feed watcher
                self.emit(SIGNAL('feedwatcher'))

        else:
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import SmewtException, SolvingChain, Media, Metadata, utils
from smewt.base.textutils import toUtf8
from smewt.media import Episode, Series, Subtitle
from smewt.taggers.tagger import Tagger
from smewt.guessers import *
from smewt.solvers import *
from pygoo import MemoryObjectGraph
import logging

log = logging.getLogger('smewt.taggers.episodetagger')

class EpisodeTagger(Tagger):

    def perform(self, query):
        log.info('EpisodeTagger tagging episode: %s' % toUtf8(query.find_one(Media).filename))
        filenameMetadata = SolvingChain(EpisodeFilename(), MergeSolver(Episode)).solve(query)
        log.info('EpisodeTagger found info from filename: %s' % filenameMetadata.find_one(Episode))
        result = SolvingChain(EpisodeTVDB(), SimpleSolver(Episode)).solve(filenameMetadata)

        media = result.find_one(Media)

        # if we didn't find a valid episode but we still have a series, let's create a syntactically
        # valid episode anyway so it can be imported
        if not media.metadata.get('episodeNumber'):
            media.metadata.episodeNumber = -1

        # import subtitles correctly
        if media.type() == 'subtitle':
            subs = []
            for language in utils.guessCountryCode(media.filename):
                log.info('Found %s sub in file %s' % (toUtf8(language), toUtf8(media.filename)))
                subs += [ result.Subtitle(metadata = media.metadata,
                                          language = language) ]

            media.metadata = subs


        self.cleanup(result)

        log.debug('Finished tagging: %s' % media.filename)
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from pygoo import MemoryObjectGraph, Equal
from smewt.base import Media, Task
import logging

log = logging.getLogger('smewt.importtask')

class ImportTask(Task):
    def __init__(self, collection, taggerType, filename):
        super(ImportTask, self).__init__()
        self.collection = collection
        self.taggerType = taggerType
        self.filename = filename

    def perform(self):
        query = MemoryObjectGraph()
        query.Media(filename = self.filename)
        result = self.taggerType().perform(query)

        # TODO: check that we actually found something useful
        #result.display_graph()

        # import the data into our collection
        self.collection.add_object(result.find_one(Media),
                                   recurse = Equal.OnUnique)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Ricard Marxer <email@ricardmarxer.com>
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import sys, os.path
from collections import *
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from smewt.base.textutils import toUtf8
from smewt.base import utils

class DirSelector(QWidget):
    def __init__(self, focusDir = QDir.currentPath(), parent = None, folders = [], recursiveSelection = True):
        QWidget.__init__(self)

        self.focusDir = focusDir
        self.model = DirModel()
        self.setSelectedFolders( folders )
        self.tree = QTreeView()
        self.tree.setModel(self.model)

        self.connect(self.model, SIGNAL('selectionChanged'),
                     self.selectionChanged)

        # Hide header and all columns but the dirname
        self.tree.header().hide()
        for i in range(self.model.columnCount()-1):
            self.tree.hideColumn(i+1)

        # Expand the first level if there is only one directory
        # Useful for Linux where / is always the first and only directory
        if self.model.rowCount() == 1:
            currentIndex = self.model.index(0,0)
            self.tree.scrollTo( currentIndex,
                                QAbstractItemView.PositionAtTop )
            self.tree.setCurrentIndex( currentIndex )
            self.tree.expand( currentIndex )

        # Don't allow selection
        self.tree.setSelectionMode(QAbstractItemView.NoSelection)

        self.recursive_checkbox = QCheckBox('Select the folders recursively')
        self.connect(self.recursive_checkbox, SIGNAL('stateChanged(int)'), self.setRecursiveSelection)

        self.recursive_checkbox.setCheckState( Qt.Checked if recursiveSelection else Qt.Unchecked )

        self.formLayout = QVBoxLayout()
        self.formLayout.addWidget(self.tree)
        self.formLayout.addWidget(self.recursive_checkbox)
        self.setLayout(self.formLayout)


    def expandPathNode(self, fullpath):
        spath = utils.splitPath(fullpath)
        for i in range(len(spath)):
            self.tree.expand(self.model.index(os.path.join(*spath[:i+1])))

    def recursiveSelection(self):
        return self.model.recursiveSelection

    def setRecursiveSelection(self, state):
        if state == Qt.Checked:
            self.model.setRecursiveSelection( True )

        elif state == Qt.Unchecked:
            self.model.setRecursiveSelection( False )

        self.selectionChanged()

    def selectionChanged(self):
        self.emit(SIGNAL('selectionChanged'))

    def setSelectedFolders(self, folders = []):
        return self.model.setSelectedFolders(folders = folders)

    def selectedFolders(self):
        return self.model.selectedFolders()


class DirModel(QFileSystemModel):

    def __init__(self, parent = None, recursiveSelection = False):
        QFileSystemModel.__init__(self, parent)
        self.setRootPath('/')
        self.setFilter( QDir.AllDirs | QDir.NoDotAndDotDot )
        self.recursiveSelection = recursiveSelection
        self.clearSelectedFolders()

    def recursiveSelection(self):
        return self.recursiveSelection

    def setRecursiveSelection(self, recursiveSelection):
        self.recursiveSelection = recursiveSelection
        for folder in self.selectedFolders():
            index = self.index(folder)
            if index.isValid():
                self.childrenDataChanged(index)

            self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
                self.index(folder),
                self.index(folder))

    def clearSelectedFolders(self):
        self.checkstates = defaultdict( lambda : Qt.Unchecked )

    def setSelectedFolders(self, folders):
        self.clearSelectedFolders()

        for folder in folders:
            index = self.index(folder)
            if index.isValid():
                self.setCheckState( index, Qt.Checked )
            else:
                self.checkstates[folder] = Qt.Checked

    def selectedFolders(self):
        return [ k for k, v in self.checkstates.items() if v == Qt.Checked ]

    def data(self, index, role = Qt.DisplayRole):
        if (role == Qt.CheckStateRole and index.column() == 0):
            if self.checkState(index) == Qt.Checked:
                return QVariant( self.checkState(index) )

            else:
                if self.recursiveSelection:
                    if self.parentChecked( index ):
                        return QVariant( Qt.Checked )

                if self.childChecked( index ):
                    return QVariant( Qt.PartiallyChecked )
                else:
                    return QVariant( Qt.Unchecked )
        return QFileSystemModel.data(self, index, role)

    def childChecked(self, index):
        for checkedKey in self.selectedFolders():
            if toUtf8(checkedKey).startswith( toUtf8(self.key( index )) + '/' ):
                return True

        return False

    def parentChecked(self, index):
        key = self.key( index )
        for checkedKey in [folder for folder in self.selectedFolders() if folder != key ]:
            if toUtf8(key).startswith(toUtf8(checkedKey) + '/'):
                return True

        return False

    def key(self, index):
        return self.fileInfo(index).absoluteFilePath()

    def setCheckState(self, index, state):
        self.checkstates[ self.key(index) ] = state


    def childrenDataChanged(self, index, recursive = False):
        self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
                  self.index(0, 0, index),
                  self.index(self.rowCount(index)-1, 0, index))

    def parentDataChanged(self, index, recursive = False):
        if recursive and self.parent( index ).isValid():
                self.parentDataChanged( self.parent( index ), recursive = recursive)
        self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"),
                  self.parent(index), self.parent(index))

    def checkState(self, index):
        return self.checkstates[ self.key(index) ]

    def setData(self, index, value, role = Qt.EditRole):
        if (role == Qt.CheckStateRole and index.column() == 0):
            state = self.checkState(index)

            if state == Qt.Checked:
                self.setCheckState(index, Qt.Unchecked)

            elif state == Qt.Unchecked:
                self.setCheckState(index, Qt.Checked)

            self.emit(SIGNAL("dataChanged(QModelIndex,QModelIndex)"), index, index)
            self.parentDataChanged(index, recursive = True)
            if self.recursiveSelection:
                self.childrenDataChanged(index, recursive = True)

            self.emit(SIGNAL("selectionChanged"))
            return True

        self.emit(SIGNAL("selectionChanged"))
        return QFilesystemModel.setData(self, index, value, role)


    def flags(self, index):
        if self.recursiveSelection:
            if self.parentChecked( index ):
                return Qt.NoItemFlags

        return Qt.ItemIsUserCheckable  | Qt.ItemIsEnabled



if __name__ == "__main__":
    app = QApplication(sys.argv)

    form = DirSelector()
    form.setWindowTitle("Test")
    form.show()
    sys.exit(app.exec_())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Smewt - A smart collection manager
# Copyright (c) 2008 Nicolas Wack <wackou@gmail.com>
#
# Smewt is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Smewt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from smewt.base import Metadata


class Subtitle(Metadata):
    '''Metadata object used for representing subtitles.

    Note: the language property should be the 2-letter code as defined in:
    http://en.wikipedia.org/wiki/ISO_3166-1_alpha-2
    '''

    typename = 'Subtitle'

    schema = { 'metadata': Metadata,
               'language': unicode }

    valid = [ 'metadata' ]

    reverse_lookup = { 'metadata': 'subtitles' }

    order = [ 'metadata', 'language' ]

    unique = [ 'metadata', 'language' ]

    converters = {}
